"""Numerical methods for mathematical functions neede for the program"""

#This method finds the factorial of a given number(num).
#The method calls for an integer input num
#The method returns the factorial(represented as fact in the method)
def factorial(num):

    """Finds the factorial of the input integer.
    
    :arg num: an integer
    """
    #If the number provided is zero then the factorial is 1
    if num == 0:
        fact = 1
    #Otherwise set fact to 1 and begin finding the factorial r is
    #used to find each num-n for n=0 to n=num each value in r is
    #then used to compute the factorial
    else:
        fact = 1
        r = list(range(1,num+1))
        for i in r:
            fact *= i
    return fact

def binomial_coefficient(n, k):

    if k < 0 or k > n:
        return 0
    if k==0 and n == 0:
        return 1
    t = 1
    if k < n-k:
        for i in range(n, n-k, -1):
            t = t*i//(n-i+1)
    else:
        for i in range(n, k, -1):
            t = t*i//(n-i+1)

    return t



"""This module contains methods used for visualizing the enuerated structures."""

from phenum.base import testmode
from matplotlib import cm
import matplotlib

import os
if os.name != "nt":
    matplotlib.use("Agg" if testmode else "TkAgg")    

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

def HNF_shapes(enum,lattice,show,testmode=False):
    """Plots the shape of each HNF.

    :arg enum: The enum.in style input file.
    :arg lattice: The lattice.in style input file.
    :arg show: If true each HNF is plotted in an interactive window.
    """
    from phenum.io_utils import read_lattice
    from phenum.grouptheory import SmithNormalForm, get_full_HNF
    from phenum.vector_utils import map_enumStr_to_real_space, cartesian2direct
    from operator import mul
    from numpy import array, mgrid, dot
    from itertools import product

    try:
        from functools import reduce
    except ImportError:
        import numpy as np
    
    lattice_data = read_lattice(lattice)
    system = _convert_read_lat_to_system_dat(lattice_data)
    
    with open(enum,"r") as inf:
        for line in inf:
            if "#" in line:
                pass
            else:
                structure = {}
                HNF_name = [int(i) for i in line.strip().split()[:-1]]
                if len(HNF_name) != 6:
                    HNF_name = HNF_name[0:6]
                structure["HNF"] = get_full_HNF(HNF_name)
                (SNF,L,R) = SmithNormalForm(structure["HNF"])
                structure["diag"] = [SNF[0][0],SNF[1][1],SNF[2][2]]
                structure["L"] = L
                structure["n"] = reduce(mul,structure["diag"],1)
                structure["labeling"] = "".join(["0" for i in range(structure["n"])])
                system["nD"] = len(system["dvecs"])
                space_data = map_enumStr_to_real_space(system,structure,True)
                space_data["aBas"] = cartesian2direct(space_data["sLV"],space_data["aBas"],system["eps"])
                # the corners of the polyhedron except the origin.
                x = space_data["sLV"][0]
                y = space_data["sLV"][1]
                z = space_data["sLV"][2]
                xy = (array(x)+array(y)).tolist()
                xz = (array(x)+array(z)).tolist()
                yz = (array(y)+array(z)).tolist()
                xyz = (array(y)+array(x)+array(z)).tolist()

                correct = [[0,0,0],x,y,z,xy,xz,yz,xyz]
                xf = [i[0] for i in correct]
                yf = [i[1] for i in correct]
                zf = [i[2] for i in correct]

                # we need to put the shifted atoms back into cartesian corrdinates.
                atoms = []
                for atom in space_data["aBas"]:
                    atoms.append(dot(atom,space_data["sLV"]).tolist())
                        
                xa = [atom[0] for atom in atoms]
                ya = [atom[1] for atom in atoms]
                za = [atom[2] for atom in atoms]

                fig = plt.figure()
                ax = fig.gca(projection='3d')
                ax.set_aspect('equal')
                ax.set_axis_off()
                ax.scatter(xa,ya,za,zdir='z',c='red',s=1000)
                # ax.scatter(xf,yf,zf,zdir='z',c='red')

                ax.plot([0,x[0]],[0,x[1]],[0,x[2]],'k')
                ax.plot([0,y[0]],[0,y[1]],[0,y[2]],'k')
                ax.plot([0,z[0]],[0,z[1]],[0,z[2]],'k')
                ax.plot([xz[0],x[0]],[xz[1],x[1]],[xz[2],x[2]],'k')
                ax.plot([xz[0],z[0]],[xz[1],z[1]],[xz[2],z[2]],'k')
                ax.plot([xy[0],x[0]],[xy[1],x[1]],[xy[2],x[2]],'k')
                ax.plot([xy[0],y[0]],[xy[1],y[1]],[xy[2],y[2]],'k')
                ax.plot([xz[0],z[0]],[xz[1],z[1]],[xz[2],z[2]],'k')
                ax.plot([yz[0],z[0]],[yz[1],z[1]],[yz[2],z[2]],'k')
                ax.plot([yz[0],y[0]],[yz[1],y[1]],[yz[2],y[2]],'k')
                ax.plot([yz[0],xyz[0]],[yz[1],xyz[1]],[yz[2],xyz[2]],'k')
                ax.plot([xz[0],xyz[0]],[xz[1],xyz[1]],[xz[2],xyz[2]],'k')
                ax.plot([xy[0],xyz[0]],[xy[1],xyz[1]],[xy[2],xyz[2]],'k')

                max_range = array([array(xf).max()-array(xf).min(),array(yf).max()-array(yf).min(),
                                   array(zf).max()-array(zf).min()]).max()
                Xb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(xf).max()+array(xf).min())
                Yb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(yf).max()+array(yf).min())
                Zb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(zf).max()+array(zf).min())

                for xb, yb, zb in zip(Xb,Yb,Zb):
                    ax.plot([xb],[yb],[zb],'w')

                if not testmode: # pragma: no cover
                    fig.savefig("{}.pdf".format("".join([str(i) for i in HNF_name])))
                if show and not testmode: #pragma: no cover
                    plt.show()
                else:
                    plt.close()
    
def HNF_atoms(enum,lattice,show,testmode=False):
    """Plots the atomic positions of the atoms in the cells.

    :arg enum: The enum.in style input file.
    :arg lattice: The lattice.in style input file.
    :arg show: If true each HNF is plotted in an interactive window.
    """
    from phenum.io_utils import read_lattice
    from phenum.grouptheory import SmithNormalForm, get_full_HNF
    from phenum.vector_utils import map_enumStr_to_real_space, cartesian2direct
    from operator import mul
    from numpy import array, mgrid, dot
    from itertools import product
    
    try:
        from functools import reduce
    except ImportError:
        import numpy as np

    lattice_data = read_lattice(lattice)
    system = _convert_read_lat_to_system_dat(lattice_data)
    
    with open(enum,"r") as inf:
        for line in inf:
            if "#" in line:
                pass
            else:
                structure = {}
                HNF_name = [int(i) for i in line.strip().split()[:-1]]
                if len(HNF_name) != 6:
                    HNF_name = HNF_name[0:6]
                structure["HNF"] = get_full_HNF(HNF_name)
                (SNF,L,R) = SmithNormalForm(structure["HNF"])
                structure["diag"] = [SNF[0][0],SNF[1][1],SNF[2][2]]
                structure["L"] = L
                structure["n"] = reduce(mul,structure["diag"],1)
                structure["labeling"] = "".join(["0" for i in range(structure["n"])])
                system["nD"] = len(system["dvecs"])
                space_data = map_enumStr_to_real_space(system,structure,True)
                space_data["aBas"] = cartesian2direct(space_data["sLV"],space_data["aBas"],system["eps"])
                atoms = []
                for atom in space_data["aBas"]:
                    atoms.append(dot(atom,space_data["sLV"]).tolist())
                        
                xf = [atom[0] for atom in atoms]
                yf = [atom[1] for atom in atoms]
                zf = [atom[2] for atom in atoms]

                fig = plt.figure()
                ax = fig.gca(projection='3d')
                ax.set_aspect('equal')
                ax.set_axis_off()
                ax.scatter(xf,yf,zf,zdir='z',c='red',s=1000)

                max_range = array([array(xf).max()-array(xf).min(),array(yf).max()-array(yf).min(),
                                   array(zf).max()-array(zf).min()]).max()
                Xb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(xf).max()+array(xf).min())
                Yb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(yf).max()+array(yf).min())
                Zb = 0.5*max_range*mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(array(zf).max()+array(zf).min())

                for xb, yb, zb in zip(Xb,Yb,Zb):
                    ax.plot([xb],[yb],[zb],'w')

                if not testmode: #pragma: no cover
                    fig.savefig("{}.pdf".format("".join([str(i) for i in HNF_name])))
                if show and not testmode:#pragma: no cover
                    plt.show()
                else:
                    plt.close()

def _convert_read_lat_to_system_dat(lattice):
    """This method converts the lattice_data dictionary returned by
    io_utils.read_lattice to the dictionary structure of io_utils.read_enum_out.

    :arg lattice: The io_utils.read_lattice style dictionary.
    """

    system_data = {
        "title": "HNF plotter",
        "bulksurf": lattice["bulk"],
        "plattice": lattice["lat_vecs"],
        "nD": 0,
        "dvecs": lattice["basis_vecs"],
        "k": lattice["nspecies"],
        "eps": 1E-10
    }

    return system_data



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
AUTHORS: Conrad W. Rosenbrock, Wiley S. Morgan (June 2015)

Classes to support the calculation of coefficients for specific terms in a product
of multinomials. Construct a product class by specifying the exponent and target term
and then add multinomials using the Product instance's append(). The coefficient is
then available from the coeff().
"""
from functools import reduce
class Sequence(object):
    """Represents an exponent-limited sequence with a single root. Here, sequence represents a
    sequence of integer values k_1, k_2...k_j that are the exponents of a single term in a multinomial.
    The root of the sequence is one of the k_i; its children become sets of sequences including
    variables to the right of i.
    """
    def __init__(self, root, possibles, i, powersum, targets, parent=None):
        """Initializes a sequence collector for a variable. 'Term' refers to a product
        of variables like x^i.y^j.z^k, then x has index 0, y has 1, etc.

        :arg root: the exponent of the variable to the left in the multinomial term.
        :arg possibles: a list of possible values for each variable in the multinomial.
        :arg i: the index of the variable being sequenced in the term.
        :arg powersum: the maximum value that the sum of exponents in the sequence is allowed to have.
        :arg parent: a Sequence instance for the variable the *left* of this one
          (i.e. has index i-1).
        """
        self._root = root
        self.used = root + (0 if parent is None else parent.used)
        self.parent = parent

        #We only keep recursively defining sequences until we run out of variables in
        #the term. Possibles is a list of possible exponents for each variable in the
        #term and has the same number of items as variables in the term.
        if i < len(targets):
            #Filter the possible values for the variable being considered based on the
            #exponent of the multinomial. When multinomials are expanded, the sum of
            #the exponents in any term must be less than the exponent on the multinomial
            #times the maximum power of any of its (unexpanded) terms.

            #We find all the possible values for this variable by ensuring that:
            # 1) it's exponent is compatible with the exponents of all variables to the left of it.
            # 2) the exponent we are suggesting is in the list of possible values for the variable.
            # 3) the exponent remains positive.
            self.kids = [Sequence(p-root, possibles, i+1, powersum, targets, self) 
                         for p in possibles if p-root >= 0
                         and p-root <= targets[i]
                         and abs(p - root) <= powersum-self.used 
                         and abs(p-self.used) % possibles[1] == 0]
        else:
            self.kids = []

        self._kidcount = None
        self.varcount = len(targets)

    @property
    def kidcount(self): #pragma: no cover
        """Returns the number of children and grandchildren to the last generation."""
        if self._kidcount is None:
            _kidcount = sum([k.kidcount for k in self.kids])
            if _kidcount == 0:
                _kidcount = len(self.kids)
        return _kidcount

    def expand(self, depth=0):
        """Recursively generates a list of all relevant sequences for this multinomial term."""
        #Iterate through the child sequences and add their variable root values if
        #the total sequence sums to the target.
        sequences = []
        for kid in self.kids:
            for seq in kid.expand(depth+1):
                #Here is where the recursion happens; we add the sequence of this variable's
                #children to the right of this root.
                sequences.append((self._root,) + seq)

        if len(self.kids) == 0:
            if depth == self.varcount-1:
                return [(self._root,)]
            else: #pragma: no cover
                return [(self._root,) + (0,)*(self.varcount-(depth+1))]
        else:
            return sequences

    def expand_noappend(self, sequences, start, varindex): #pragma: no cover
        """Implements an expansion that doesn't use python's append."""
        if len(sequences) == 0:
            if self.kidcount == 0:
                raise ValueError("This can't happen!")
            sequences = [[0]*self.varcount for i in range(self.kidcount)]

        cursor = start
        for kid in self.kids:
            kid.expand_noappend(sequences, cursor, varindex+1)
            cursor += kid.kidcount

            if varindex == self.varcount-1:
                cursor += 1

            for k in range(start, cursor):
                sequences[k][varindex-1] = self._root
            start = cursor

        if self.kidcount == 0:
            sequences[cursor][varindex-1] = self._root
            
        return sequences

class Product(object):
    """Represents a product of multinomials for which only a single term is interesting."""
    def __init__(self, coefficient, targets):
        """Initializes the empty product of multinomials.

        :arg coefficient: the scalar integer multiplying this product of multinomials.
        :arg targets: a list of exponents for the only interesting term in the product. The
          list is in the order that the variables appear in each multinomial.
        """
        self.coefficient = coefficient
        self.targets = targets
        self.multinoms = []

    def coeff(self):
        """Returns the coefficient of the term with the target exponents if all the multinomials
        in the product were expanded and had their terms collected.
        """
        #If this is an isolated multinomial, we only need to check the coefficient of the target
        #term.
        if len(self.multinoms) == 1:
            if all([self.multinoms[0].power-t>0 for t in self.targets]):
                return 0
            else:
                return self.multinoms[0].nchoosekm(self.targets)*self.coefficient
        
        from itertools import product
        #Get a list of the possible exponents for each variable in each of the multinomials.
        #We start with the first variable and choose only those combinations of exponents
        #across *all* the multinomials that give the correct target exponent for that variable.
        possibles = [n.possible_powers for n in self.multinoms]
        
        seq0 = [s for s in product(*possibles) if sum(s) == self.targets[0]]
        #Next, we construct Sequence instances for each of the first variable compatible
        #possibilities and follow them through to the other variables.
        coeffs = 0
        for seq in seq0:
            mnseq = []
            #Each sequence calculated from the first variable has an entry for each multinomial
            #in this product. The Sequence instances construct smart sequences for the remaining
            #variables in each multinomial separately
            for i in range(len(seq)):
                varseq = Sequence(seq[i], possibles[i], 1, self.multinoms[i].powersum, self.targets)
                mnseq.append(varseq.expand())
            coeffs += self._sum_sequences(mnseq)

        return int(coeffs)*self.coefficient

    def _sum_sequences(self, mnseq):
        """Sums all the possible combinations of relevant sequences based of the variable sequence
        lists specified.

        :arg mnseq: a list of possible variable sequences in each multinomial (one for each multinomial)
          that might contribute to the correct target variable.
        """
        from itertools import product
        from operator import mul
        from functools import reduce
        #We can also filter the sequences by enforcing the constraints that the exponents
        #correctly reproduce the target across all the multionmials in the product. Get hold
        #of all the combinations of sequences across the multinomials and check each for
        #conformance to the targets.
        coeffs = 0
        for seq in product(*mnseq):
            expsum = [sum(zs) for zs in zip(*seq)]
            if expsum == self.targets:
                coeffs += reduce(mul, [m.nchoosekm(s) for m, s in zip(self.multinoms, seq)])

        return coeffs

    def __str__(self):
        #First we need to sort the multinomials by their exponent.
        sortedmns = sorted(self.multinoms, key=(lambda m: (m.exponent,m.power)), reverse=True)
        return str(self.coefficient) + ''.join([str(mn) for mn in sortedmns])            

class Multinomial(object):
    """Represents a multinomial expansion."""
    def __init__(self, power, coeff, arrowings, exponent=1):
        """Sets up the multinomial.

        :arg powers: the power on each of the *unexpanded* variables in the multinomial;
          of the form (x^2+y^2+z^2) => 2.
        :arg coeff: the coefficient on each of the variables in the multinomial; e.g.
          (x^2 + a y^2) => a. For more than two variables, each color that *has* arrowing
          gets the coefficient while the others get 1.
        :arg exponent: the exponent of the entire multinomial.
        """
        self.power = power
        self.coeff = coeff
        self.arrowings = arrowings
        self.exponent = exponent
        self.powersum = power*exponent
        """Returns the integer value that all term exponents in the multinomial should
        sum to (or be less than)."""
        self.possible_powers = list(range(0,power*exponent+1, power))
        """For each variable being considered, determines the possible powers based
        on the exponent in the multinomial."""

    def __str__(self):
        #We want to print the multinomial out in a nice, readable way, similar to how
        #they are presented in Mathematica.
        contents = ' + '.join(["{}^{}".format(self.coeff if a else 1, self.power) for a in self.arrowings])
        return "({})^{}".format(contents, self.exponent)

    def normed_seq(self, seq):
        """Normalizes the specified sequence using the powers of unexpanded terms in the multinomial.
        
        :arg seq: a list of exponents in an *expanded* term.
        """
        return [int(ai/self.power) for ai in seq]

    def nchoosekm(self, sequence):
        """Returns the number of different ways to partition an n-element
        set into disjoint subsets of sizes k1, ..., km.

        :arg sequence: an un-normed tuple of form (k1, k2, k3).
        """
        prod = 1
        if not all([seq%self.power == 0 for seq in sequence]):
            return 0
        else:
            from operator import mul
            normseq = self.normed_seq(sequence)
            for i in range(len(sequence)):
                nsum = sum(normseq[0:i+1])
                prod *= Multinomial.nchoosek(nsum, normseq[i])

            #Add the contribution from the coefficients of the variable *inside*
            #the multionomial.
            pcoeff = 1
            for iseq, arrow in enumerate(self.arrowings):
                if arrow:
                    pcoeff *= self.coeff**normseq[iseq]

            return prod*pcoeff
        
    @staticmethod
    def nchoosek(n, k):
        """This implementation was taken from "Binomial Coefﬁcient Computation: Recursion 
        or Iteration?" by Yannis Manolopoulos, ACM SIGCSE Bulletin InRoads, Vol.34, No.4, 
        December 2002. http://delab.csd.auth.gr/papers/SBI02m.pdf It is supposed to be robust 
        against large, intermediate values and to have optimal complexity.
        """
        if k < 0 or k > n: #pragma: no cover
            return 0
        if k==0 and n == 0:
            return 1
        t = 1
        if k < n-k:
            for i in range(n, n-k, -1):
                t = t*i/(n-i+1)
        else:
            for i in range(n, k, -1):
                t = t*i/(n-i+1)

        return t

def group(gen): #pragma: no cover
    """Generates an entire group using the specified generators by applying generators
    to each other recursively.

    :arg gen: a list of generators as integers.
    """
    from operator import itemgetter as iget
    def g_apply(operations, source, groupi=None):
        """Applies the specified group operations to the source list of elements and then
        appends it to the group if it is unique.
        """
        result = list(iget(*operations)(source))
        if groupi is not None and result not in groupi:
            groupi.append(result)
        return result

    #Make sure the group is zero-based for python.
    if not 0 in gen[0]:
        ngens = [list([e-1 for e in g]) for g in gen]
    else:
        ngens = gen

    groupi = []
    for i in ngens:
        for j in ngens: #filter(lambda k: k!=i, ngens):
            c = g_apply(i, j, groupi)
            d = g_apply(i, c, groupi)
            while d != c:
                d = g_apply(i, d, groupi)

    while True:
        group2 = []
        for i in ngens:
            for h in groupi:
                d = g_apply(i, h)
                if d not in groupi and d not in group2:
                    group2.append(d)

        groupi.extend(group2)
        if len(group2) == 0:
            break
    return(groupi)

def _group_to_cyclic(group, limit=None):
    """Determines the degeneracy of each r-cycle in the specified group operations."""
    result = []
    #We allow filtering so that the unit testing can access the cyclic form of the group.
    if 0 not in group[0][0]: #pragma: no cover
        group = [[[j - 1 for j in i] for i in t] for t in group]
    
    if limit is not None: #pragma: no cover
        filtered = group[limit[0]:limit[1]]
    else:
        filtered = group

    for operation in filtered:
        #visitedp has the same # of elements as the site group operation and
        #is used to make sure each element in the array is visited as
        #we loop through in a *non-sequential* order.
        #visitedc has the same number of elements as the arrow group
        #opertaion and is used to make sure each element of that array
        #is visited as we loop through it in a "non-sequential" order.
        visitedp = [0]*len(operation[0])
        visitedc = [0]*len(operation[1])
        polynomials = {}
        
        arrow_cycle_len = []
        #We start by finding the cylce lengths of the arrow group
        #operations so that we can later see how many of them are
        #divosors of the site group operations.
        while 0 in visitedc:
            #Start with the first elemenet in the arrow group that
            #hasn't been visited yet. All cycles have length > 0.
            cursor = vindex = visitedc.index(0)
            cyc_len = 1
            visitedc[cursor] = 1
            cursor = operation[1][cursor]
            while cursor != vindex:
                visitedc[cursor] = 1
                cyc_len += 1
                cursor = operation[1][cursor]
            arrow_cycle_len.append(cyc_len)

        while 0 in visitedp:
            #Start with the first element in the site group that hasn't
            #been visited yet. The first non-trivial polynomials have
            #powers > 0.
            cursor = vindex = visitedp.index(0)
            powers = 1 
            #change the current position to having been visited; move the cursor.
            visitedp[cursor] = 1
            cursor = operation[0][cursor]
            #The power of the variables in the polynomials is equal to
            #the number of group operations separating the cursor's
            #current position from its *value* in the group operations
            #list.
            while cursor != vindex:
                visitedp[cursor] = 1
                powers += 1
                cursor = operation[0][cursor]
            #We now have everything need to construct part of the
            #polynomial. This is done by taking powers and using it to
            #construct an array of length equal to the number of
            #elements in the system each entry in the array is set to
            #be equal to powers.

            #To get the coefficients right we need to see how many of
            #the arrow cycles are divisors in length of the current
            #cycle.
            coef = sum([l for l in arrow_cycle_len if powers%l == 0])
            polykey = (powers, coef)
            if polykey not in polynomials:
                polynomials[polykey] = 1
            else:
                polynomials[polykey] += 1
        result.append(polynomials)
        
    return result

def polya(concentrations, group, arrowings=None, debug=False):
    """Uses a group and concentrations to find the number of unique arrangements as described by 
    polya.
    
    :arg concentrations: specify a list of integers specifying how many of each coloring should
      be present in each of the enumerated lists.
    :arg group: group operations for permuting the colorings.
    """
    temp = [str(i) for i in concentrations]
    for i in group:
        temp = [str(j) for j in i[0]]

    for i in group:
        temp = [str(j) for j in i[1]]
    
    if arrowings is None:
        arrowings = [False]*len(concentrations)
    elif isinstance(arrowings, int):
        arrowings = [False]*(len(concentrations)-arrowings) + [True]*arrowings
        
    #This is to check that the concentrations sum to the number of sites the group is
    #operating on
    
    if sum(concentrations) != len(group[0][0]):
        raise ValueError("The concentrations don't sum to the number of things the group is acting on!")
            
    for k in range(2):
        for g in range(len(group)):
            if 0 not in group[g][k]:
                group[g][k] = [j-1 for j in group[g][k]]

    polyndict = {}
    #The operations in the group are used to construct the unique polynomials for each operation.
    for polynomials in _group_to_cyclic(group):
        #Construct a product of multinomials for this group operation.
        p = Product(1,concentrations)
        for exp, coeff in polynomials:
            p.multinoms.append(Multinomial(exp, coeff, arrowings, polynomials[(exp, coeff)]))

        key = str(p)
        if key not in polyndict:
            polyndict[key] = p
        else:
            polyndict[key].coefficient += 1

    if debug: #pragma: no cover
        for key in polyndict:
            print((str(polyndict[key]), " => ", polyndict[key].coeff()))
            
    pp = 0
    for p in list(polyndict.values()):
        pp += p.coeff()
        
    rad = sum([p.coeff() for p in list(polyndict.values())])
    return int(rad/float(len(group)))



"""Contains routines used to map the enumerated structures to real space."""

def map_enumStr_to_real_space(system_data,structure_data,minkowskiReduce):
    """Maps an enumerated structure back to real space. Returns a
    dictionary containing the real space data.

    :arg system_data: a dictionary containing all the information about the sysytem.
    :arg sturture_data: a dictionary containing the information for this structure.
    :arg minkowskiReduce: logical indicating if basis should be reduced.

    """
    # print("in system_data",system_data)
    # print("in structure_data",structure_data)
    # print("in minkReduce",minkowskiReduce)
    from numpy import matmul, allclose, matrix, array
    
    nD = system_data["nD"]
    n = structure_data["n"]

    # DEFINE the non-zero elements of the HNF matrix
    a = structure_data["HNF"][0][0]
    b = structure_data["HNF"][1][0]
    c = structure_data["HNF"][1][1]
    d = structure_data["HNF"][2][0]
    e = structure_data["HNF"][2][1]
    f = structure_data["HNF"][2][2]

    pBas = system_data["dvecs"]
    S = structure_data["diag"]
    pLV = system_data["plattice"]
    HNF = structure_data["HNF"]
    eps = system_data["eps"]
    L = structure_data["L"]
    # Compute the superlattice vectors
    sLV = matmul(pLV,HNF).tolist()
    # Find the coordinates of the basis atoms
    gIndx = []

    if minkowskiReduce:
        sLV = list(map(list,zip(*_minkowski_reduce_basis(list(map(list,zip(*sLV))),eps))))
        
    # Find each atomic position from the g-space information
    aBas = []
    ic = 0  # Keep track of the number of points mapped so far
    # Loop over the number at sites/parent cell (the d set)it
    for iD in range(0, nD):  
        # For the limits on the loops, see the "interior_points.pdf" write-up
        for z1 in range(a):
            for z2 in range(int((b*z1)/a), int(c+(b*z1)/a)):
                for z3 in range(int(z1*(d-(e*b)/c)/a+(e*z2)/c), int(f+z1*(d-(e*b)/c)/a+(e*z2)/c)):
                    ic +=1
                    if ic > n: #pragma: no cover
                        from .msg import err
                        err("Problem with basis atoms in map_enpStr_to_real_space...")
                        exit()
                    # Atomic basis vector in Cartesian coordinates
                    temp = matmul(pLV,[z1,z2,z3]).tolist()
                    temp2 = [temp[i]+pBas[iD][i] for i in range(len(pBas[iD]))]
                    aBas.append(temp2)
                    # Map position into the group
                    greal = matmul(L,[float(z1),float(z2),float(z3)]).tolist() 
                    g = [int(i) for i in greal] # Convert the g-vector from real to integer
                    if not allclose(greal,g,rtol=eps,atol=eps): #pragma: no cover
                        from .msg import err
                        err("map2G didn't work in map_enumStr_to_real_space")
                        exit()
                    # Bring the g-vector back into the first tile
                    g = [g[i]%S[i] for i in range(len(S))] 
                    # gIndx is the index in the configuration string that
                    # tells us which atom type is used at this position

                    gIndx.append((iD)*S[0]*S[1]*S[2]+g[0]*S[1]*S[2]+g[1]*S[2]+g[2])
    if ic != n*nD: #pragma: no cover
        from .msg import err
        err("ERROR: map_enumStr_to_real_space: Didn't find the correct # of basis atoms")
        exit()

    k = system_data["k"]
    x = []
    for i in range(k):
        x.append(0.0)
        
    labeling = structure_data["labeling"]
    spin = []
    if k % 2 == 0:
        for iAt in range(0, n*nD):
            i = int(labeling[gIndx[iAt]])
            digit = i-k//2 # convert 0..k-1 label to spin variable -k/2..k/2
            x[i] += 1  # Keep track of the concentration of each atom type
            if digit < 0:
                spin.append(digit)
            else:
                spin.append(digit+1) # skip 0 as a spin if k is even
    else:
        for iAt in range(0, n*nD):
            i = int(labeling[gIndx[iAt]])
            spin.append(i-k//2)
            x[i] += 1 # Keep track of the concentration of each atom type

    x = [i/float(n*nD) for i in x]

    space_data = {"sLV": list(map(list,zip(*sLV))), "aBas": aBas, "spin": spin, "gIndx": gIndx, "x": x}
    # print("out space_data",space_data)
    return space_data


def _minkowski_reduce_basis(IN,eps):
    """Performs a minkowski reduction on the basis atoms."""

    from numpy import allclose, linalg, array, matrix
    from copy import deepcopy

    limit = 10

    if allclose(linalg.det(IN),0.0,rtol=eps,atol=eps):
        raise ValueError("Input basis for 'minkowski_reduce_basis' was not linearly independent")

    OUT = deepcopy(IN)

    # Keep applying the greedy algorithm until the vectors come out already sorted
    for it in range(1, limit +1):
        # Sort the three vectors into ascending order
        temp = deepcopy(OUT)
        norms = linalg.norm(temp,axis=1).tolist()
        tt = list(range(3))
        tt.reverse()
        for i in tt:
            idx = norms.index(max(norms))
            temp[i] = OUT[idx]
            norms[idx] = 0

        OUT = deepcopy(temp) # Copy the sorted vectors back to OUT
        (OUT[0], OUT[1], OUT[2]) = _reduce_C_in_ABC(OUT[0],OUT[1],OUT[2],eps)
        if linalg.norm(OUT[2]) >= (linalg.norm(OUT[1])-eps):
            break

    if not _minkowski_conditions_check(OUT,eps): #pragma: no cover
        from .msg import err
        err("ERROR in minkowski_reduce_basis: Minkowski conditions not met."
            "Number of iterations: {}".format(str(limit)))
        exit()
    
    # we want to make sure that the det is positive.
    # NOTE: This *destroys* the mathematical picture of a "greedy reduced basis" (Minkowski), but
    #       from a physical point of view we don't care ;-)
    #       Either way, the basis is as orthogonal as possible.
    if linalg.det(OUT) < 0:
        temp[0] = OUT[1]
        OUT[1] = OUT[2]
        OUT[2] = temp[0]

    return OUT

def _minkowski_conditions_check(basis,eps):
    """This function checks the minkowski conditions for a 3D lattice
    basis.

    :arg basis: The atomic basis vectors
    :arg eps: finitie precision tolerance
    """
    from numpy import linalg
    from .msg import err
    
    b1 = basis[0]
    b2 = basis[1]
    b3 = basis[2]

    minkowski_check = True
    if linalg.norm(b1) > (linalg.norm(b2) + eps):
        minkowski_check = False
        err("Minkowski_condition 1 failed: b1 > b2")

    if linalg.norm(b2) > (linalg.norm(b3) + eps):
        minkowski_check = False
        err("Minkowski_condition 2 failed: b2 > b3")

    if linalg.norm(b2) > (linalg.norm([b1[i]+b2[i] for i in range(len(b1))])+eps):
        minkowski_check = False
        err("Minkowski_condition 3 failed: b2 > b1+b2")

    if linalg.norm(b2) > (linalg.norm([b1[i]-b2[i] for i in range(len(b1))])+eps):
        minkowski_check = False
        err("Minkowski_condition 4 failed: b2 > b1-b2")

    if linalg.norm(b3) > (linalg.norm([b1[i]+b3[i] for i in range(len(b1))])+eps):
        minkowski_check = False
        err("Minkowski_condition 5 failed: b3 > b1+b3")

    if linalg.norm(b3) > (linalg.norm([b3[i]-b1[i] for i in range(len(b1))])+eps):
        minkowski_check = False
        err("Minkowski_condition 6 failed: b3 > b3-b1")

    if linalg.norm(b3) > (linalg.norm([b2[i]+b3[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 7 failed: b3 > b2+b3")

    if linalg.norm(b3) > (linalg.norm([b3[i]-b2[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 8 failed: b3 > b3-b2")

    if linalg.norm(b3) > (linalg.norm([b1[i]+b2[i]+b3[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 9 failed: b3 > b1+b2+b3")

    if linalg.norm(b3) > (linalg.norm([b1[i]-b2[i]+b3[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 10 failed: b3 > b1-b2+b3")

    if linalg.norm(b3) > (linalg.norm([b1[i]+b2[i]-b3[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 11 failed: b3 > b1+b2-b3")

    if linalg.norm(b3) > (linalg.norm([b1[i]-b2[i]-b3[i] for i in range(len(b2))])+eps):
        minkowski_check = False
        err("Minkowski_condition 12 failed: b3 > b1-b2-b3")

    return minkowski_check

def _reduce_C_in_ABC(A,B,C,eps):
    """This routine takes three vectors, A,B,C, defining a lattice, and
    reduces the last one so that it is as close as possible to the
    origin while remaining in an affine plane, which is parallel to
    the A-B plane but which passes through the end of the C
    vector. See Lecture notes in computer science, ISSN 0302-974, ANTS
    - VI : algorithmic number theory, 2004, vol. 3076, pp. 338-357
    ISBN 3-540-22156-5

    :arg A: a vector
    :arg B: a vector
    :arg C: a vector
    :arg eps: finite precision tolerance
    """
    from numpy import cross, linalg, dot, allclose, matmul, array
    from copy import deepcopy
    from math import floor
    
    oldABC = deepcopy([A,B,C])
    
    # Use Gaussian reduction to reduce the A,B 2D basis so that it is
    # itself Minkowski reduced. If this is done, then the closest
    # lattice point (in A,B plane) to the projection of C (into the
    # A,B plane) is guaranteed to be one of the corners of the unit
    # cell enclosing the projection of C
    (A,B) = _gaussian_reduce_two_vectors(A,B,eps)

    # First thing to do is find the (real, not lattice) point in the
    # affine plane A,B + C that is nearest the origin. Call this T.
    cpdAB = [i/linalg.norm(cross(A,B)) for i in cross(A,B)]
    T = [C[i] - cpdAB[i]*dot(C,cpdAB) for i in range(3)]

    if not allclose(dot(T,cross(A,B)),0,atol=eps,rtol=eps): #pragma: no cover
        from .msg import err
        err("{} Projection of C into A,B plane failed".format(str(dot(T,cross(A,B)))))

    # Now find the four points of the A,B lattice, in the affine
    # plane, that enclose the point T
    ABC = [A,B,C]
    ABCinv = linalg.inv(ABC)

    LC = [int(floor(i +eps)) for i in matmul(T,ABCinv).tolist()]
    # Compute the distance from T to each of the four corners of the cell and pick
    # the one that is the closest.
    corners = array([[0,0,0],[1,0,0],[0,1,0],[1,1,0]])
    dist = []
    for i in range(0,4):
        temp1 = corners[i] + array(LC)
        temp2 = array(T) -matmul((corners[i] + array(LC)),ABC)
        dist.append(linalg.norm(array(T) -matmul((corners[i] + array(LC)),ABC)))

    idx = dist.index(min(dist))

    if idx == 0:
        temp1 = [corners[0][i] + LC[i] for i in range(3)]
        temp2 = matmul(temp1,ABC).tolist()
        C = [C[i] - temp2[i] for i in range(len(C))]
    elif idx == 1:
        temp1 = [corners[1][i] + LC[i] for i in range(3)]
        temp2 = matmul(temp1,ABC).tolist()
        C = [C[i] - temp2[i] for i in range(len(C))]
    elif idx == 2:
        temp1 = [corners[2][i] + LC[i] for i in range(3)]
        temp2 = matmul(temp1,ABC).tolist()
        C = [C[i] - temp2[i] for i in range(len(C))]
    elif idx == 3:
        temp1 = [corners[3][i] + LC[i] for i in range(3)]
        temp2 = matmul(temp1,ABC).tolist()
        C = [C[i] - temp2[i] for i in range(len(C))]
    else: #pragma: no cover
        from .msg import err 
        err("Case failed in reduce_C_in_ABC"
            "Lattice coordinates in the A,B plane: ".format(' '.join([str(i) for i in LC])))

    ABC = [A,B,C]
    ABCinv = linalg.inv(ABC)
    temp = matmul(list(map(list,zip(*ABCinv))),list(map(list,zip(*oldABC)))).tolist()
    for i in range(3):
        for j in range(3):
            if abs(temp[i][j] - int(round(temp[i][j]))) > eps: #pragma: no cover
                from .msg import err
                err("Lattice was not preserved in reduce_C_in_ABC")
                exit()

    return A, B, C

def _gaussian_reduce_two_vectors(U,V,eps):
    """This routine takes two vectors (in three-space) and reduces them to
    form a shortest set (Minkowski reduced). The idea is to subtract
    multiples of U from V so that the new V is as close to the origin
    as any lattice point along the line that passes through U in the
    direction of V. The process is repeated until the new vector isn't
    shorter than the other. It's pretty obvious if you do an example
    by hand. Also see 3.1 of Lecture notes in computer science, ISSN
    0302-974, ANTS - VI: algorithmic number theory, 2004, vol. 3076,
    pp. 338-357 ISBN 3-540-22156-5. Fixes made Apr 2012 GLWH (not sure
    if they made a practical difference though)

    :arg U: a vector
    :arg V: a vector
    :arg eps: finite precision tolerance
    """

    from numpy.linalg import norm
    from numpy import dot

    it = 0
    if norm(U) > (norm(V) - eps):
       # Make sure that the {U,V} are listed in ascending order; ||U||<||V||
       temp = U
       U = V
       V = temp # Keep V as the longest vector

    done = False
    it = 1
    while not done:
        if it > 10: # pragma: no cover
            from .msg import err
            err("gaussian_reduce_two_vectors failed to converge in 10 iterations")
            exit()
        R = [V[i]-int(round(dot(U,V)/dot(U,U)+1E-10))*U[i] for i in range(3)] #Shorten V as much as possible
        V = U # Swap U and V (so U remains the shortest)
        U = R
        if norm(U) >= (norm(V) - eps):
            done = True
        it += 1

    # Make sure that the {U,V} are listed in ascending order on exit; ||U||<||V||
    temp = U
    U = V
    V = temp
    return U, V

def cartesian2direct(sLV,aBas, eps):
    """This routine takes three lattice vectors and a list of atomic basis
    vector in Cartesian coordinates and converts them to direct
    ("lattice") coordinates.

    :arg sLV: Superlattice vectors (Cartesian coordinates)
    :arg aBas: Atomic positions (cartesian coordinates first, then direct)
    :arg eps: Finite precision tolerance.

    """
    from numpy import linalg, matmul, array
    
    nAt = len(aBas)
    sLVinv = linalg.inv(sLV)
    # Convert aBas to DIRECT COORDINATES
    for iAt in range(nAt):
        aBas[iAt] = matmul(aBas[iAt],sLVinv) # Put positions into
        # "direct" coordinates This keeps the atomic coordinates inside
        # the first unit cell--- not necessary but aesthetically
        # pleasing.

        while any(aBas[iAt] >= (1.0-eps)) or any(aBas[iAt] < (0.0-eps)):
            aBas[iAt] = array([i if i < (1.0-eps) else i-1.0 for i in aBas[iAt]])
            aBas[iAt] = array([i if i >= (0.0-eps) else i+1.0 for i in aBas[iAt]])

    return aBas



import warnings
def deprecated(func):
    '''This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.'''
    def new_func(*args, **kwargs):
        warnings.warn("Call to deprecated function {}.".format(func.__name__),
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func

def exhandler(function, parser):
    """If -examples was specified in 'args', the specified function
    is called and the application exits.

    :arg function: the function that prints the examples.
    :arg parser: the initialized instance of the parser that has the
      additional, script-specific parameters.
    """
    args = vars(bparser.parse_known_args()[0])
    if args["examples"]:
        function()
        return
    if args["verbose"]:
        from phenum.msg import set_verbosity
        set_verbosity(args["verbose"])

    args.update(vars(parser.parse_known_args()[0]))
    return args

def _common_parser():
    """Returns a parser with common command-line options for all the scripts
    in the fortpy suite.
    """
    import argparse
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("-examples", action="store_true",
                        help="See detailed help and examples for this script.")
    parser.add_argument("-verbose", action="store_true",
                        help="See verbose output as the script runs.")
    parser.add_argument('-action', nargs=1, choices=['save','print'], default='print',
                        help="Specify what to do with the output (print or save)")
    parser.add_argument("-debug", action="store_true",
                        help="Print verbose calculation information for debugging.")

    return parser

bparser = _common_parser()
testmode = False
"""bool: when True, the package is operating in unit test mode, which changes
how plotting is handled.
"""
def set_testmode(testing):
    """Sets the package testing mode.
    """
    global testmode
    testmode = testing



# The dictionary of all the elements on the periodic table
all_elements ={"H": 3.75,"He": 3.57,"Li": 3.49,"Be": 2.29,"B": 8.73,"C": 3.57,"N": 4.039,
               "O": 6.83,"Ne": 4.43,"Na": 4.23,"Mg": 3.21,"Al": 4.05,"Si": 5.43,"P": 7.17,
               "S": 10.47,"Cl": 6.24,"Ar": 5.26,"K": 5.23,"Ca": 5.58,"Sc": 3.31,"Ti": 2.95,
               "V": 3.02,"Cr": 2.88,"Mn": 8.89,"Fe": 2.87,"Co": 2.51,"Ni": 3.52,"Cu": 3.61,
               "Zn": 2.66,"Ga": 4.51,"Ge": 5.66,"As": 4.13,"Se": 4.36,"Br": 6.67,"Kr": 5.72,
               "Rb": 5.59,"Sr": 6.08,"Y": 3.65,"Zr": 3.23,"Nb": 3.3,"Mo": 3.15,"Tc": 2.74,
               "Ru": 2.7,"Rh": 3.8,"Pd": 3.89,"Ag": 4.09,"Cd": 2.98,"In": 4.59,"Sn": 5.82,
               "Sb": 4.51,"Te": 4.45,"I": 7.27,"Xe": 6.2,"Cs": 6.05,"Ba": 5.02,"Hf": 3.2,
               "Ta": 3.31,"W": 3.16,"Re": 2.76,"Os": 2.64,"Ir": 3.84,"Pt": 3.92,"Au": 4.08,
               "Hg": 2.99,"Tl": 3.46,"Pb": 4.95,"Bi": 4.75}

def get_lattice_parameter(elements, concentrations, default_title):
    """Finds the lattice parameters for the provided atomic species using Vagars law.

    :arg elements: A dictionary of elements in the system and their concentrations.
    :arg title: The default system title.
    :arg concentrations: The concentrations of each element.
    """

    if elements == None:
        lat_param = 1.0
        title = default_title
    else:
        if len(elements) != len(concentrations):
            raise ValueError("You have provided {} element names when {} elements are present "
                "in the system. Please provide the correct number of elements."
                .format(len(elements),len(concentrations)))

        else:
            title = ""
            lat_param = 0
            for i in range(len(elements)):
                lat_param += concentrations[i]*all_elements[elements[i]]
                if concentrations[i] > 0:
                    title += " {} ".format(elements[i])
            lat_param = float(lat_param) / sum(concentrations)
            title = "{0} {1}\n".format(default_title.strip(),title)
    return lat_param, title



"""Methods needed for the enumeration of arrows."""

#import neede modules
from numpy import array, dot
#Find concentration takes a 1D array, col, and returns a 1D array, 
#Concs, containing the number of times each element in col appears 
#in the array.
def _find_concentrations(col):
    """Finds the concentration of a given array of colors.

    :arg col: an integer array of the labeling
    """
    Concs=[]

    tcol= list(col)
    for i in range(len(tcol)):
        if tcol[i] != 0:
            Concs.append(tcol.count(tcol[i]))
            for j in range(len(tcol)):
                if tcol[j] == tcol[i] and i != j:
                    tcol[j] =0
            tcol[i] =0
    return(Concs)

#Takes an array of colors and arrows and tells us how many different
#colors have arrows on them.
def how_many_arrows(tcol):
    """Determines the number of colors that have arrows on them.

    :arg tcol: a 2D array of the labeling that contains the colors
    and arrows for each site.
    """
    arrows = 0
    
    species = []
    for i in tcol:
        if i[0] >= 0:
            arrows += 1
            if i[1] not in species:
                species.append(i[1])

    aconcs = _find_concentrations(tcol)
    return(arrows,len(species),aconcs)

#An algorithm for sorting the col array so that colors without arraws
#appear first in the list by concentration followed by the colors with
#arraws that are also sorted by concentration.
def _col_sort(col_list):
    """Sorts the labeling so that colors with arrows appear last in the
    list and so that the arrowed and non-arrowed colors are sorted
    from lowest to highest concentration.

    :arg col_list: a 2D integer array of the full labeling of the
    system
    """

    col1 = []
    col2 = []
    colt=[]
    
    # seperate the arrays into colors with arrrows and colors without
    for i in col_list:
        if i[0] < 0:
            col1.append(i)
        elif i[0] >= 0:
            col2.append(i)

    # sort each array by concentration
    col1 = sorted(col1, key = col1.count)
    col2 = sorted(col2, key = col2.count)
    # put them back together again
    for i in col1:
        colt.append(i)
    for i in col2:
        colt.append(i)
    return(colt)

#this method takes a configuration and finds the unique ways of
#placing the arrows on that configuration.
#col is the initial configuration.
#agroup is the group operations with their effects on the arrows.
#dim is the number of arrow directions that are possible for this system.
def add_arrows(col,agroup,dim, translations, accept=None,nested=False,num_wanted=None, small=False, supers = False):
    """Finds the unique arrangements of arrows for a given configuration.

    :arg col: a 2D integer array of the initial labeling
    :arg agroup: the stabilizers for the colors only with the arrow
    permutations
    :arg dim: the number of directions the arrows can point
    :arg accept: acceptance rate of configurations for large enumerations.
    :arg nested: set to True if this is called from within the context
      of another progress bar.
    :arg supers: True if we want to include super periodic arrangements in the enumeration.
    :arg translations: The translations of the lattice.
    """
    from random import random
    #Find out how many arrows there are in the col array
    (narrows,arrow_types,conc_w_arrows) = how_many_arrows(col)
    #largest_arrow is largest value any arrow could have.
    largest_arrow = [dim-1]*narrows
    #this is an array for storing the unique arrangements.
    arsurvivors = []

    maxpossible = _ahash(largest_arrow,dim)
    from phenum.msg import verbosity
    if verbosity is not None and verbosity >= 1 and not nested: #pragma: no cover
        from tqdm import tqdm 
        #If accept is specified, then we should get on average that many out.
        if num_wanted is not None:
            ntotal = num_wanted
        else:
            ntotal = maxpossible
        pbar = tqdm(total=ntotal)

    #this loop runs over every possible configuration of arrows
    #staring from the one with the smallest hash number and ending
    #with the one with the largest hash number to see if they are
    #unique. Each hash will be stored in orighash (original hash) then
    #compared to the permuted arrows hash (permhash) at the end.
    orighash = 0
    if not small:
        while orighash <= maxpossible:
            # for orighash in range(maxpossible+1):
            arrow_config = _ainvhash(orighash,narrows,dim)
            unique = True
            temp_coloring_with_arroms = [0]*len(col)
            #we need to cycle through each of the operations in the
            #permutation group that got passed in to see if they will turn
            #the arrow configuration into one we've already seen.
            for perm in agroup:
                l = 0
                #this loop constucts the coloring with the correct arrows
                #for the permutation group to be applied to.
                for z in range(len(col)):
                    if col[z][0] < 0:
                        temp_coloring_with_arroms[z] = list(col[z])
                    elif col[z][0] >= 0:
                        temp_coloring_with_arroms[z] = list(col[z])
                        temp_coloring_with_arroms[z][0] = arrow_config[l]
                        l += 1

                new_arrow_config=[]
                arrow_perm = perm[1]
                color_perm = perm[0]
                new_coloring_with_arrows = []
                #here we use the permutations of the colors to permute the
                #colors for our configuration.
                new_coloring_with_arrows.extend([temp_coloring_with_arroms[color_perm[j]]
                                                 for j in range(len(temp_coloring_with_arroms))])
                #if there is an arrow on any lattice site it needs to
                #be updated. Otherwise just permute the colors.
                new_arrow_config.extend([arrow_perm[site[0]] for site in new_coloring_with_arrows
                                         if site[0] >= 0])
                #if the new configuration has a smaller hash number than
                #the current configuration then we have seen it before and
                #it is not unique.
                permhash = _ahash(new_arrow_config,dim)
                if permhash < orighash:
                    unique = False
                    break
                #if none of the permutations have had a smaller hash number
                #than the current configuration then the configuration is
                #unique and we need to add it to the list.
            if unique == True:
                #this loop make the coloring with the arrows that will be
                #added to the list of unique configurations.
                coloring_with_arrows = [0]*len(col)
                i = 0
                for z in range(len(col)):
                    coloring_with_arrows[z] = list(col[z])
                    if col[z][0] >= 0:
                        coloring_with_arrows[z][0] = arrow_config[i]
                        i += 1

                # We need to check the superperiodic structures.
                if not supers:
                    for trans in translations:
                        action = trans[0]
                        permutation = trans[1]
                        if action != list(range(len(action))) or permutation != list(range(len(permutation))):
                            orig_sites = [i[1] for i in coloring_with_arrows]
                            arrow_sites = [i[0] for i in coloring_with_arrows]
                            perm_sites = [orig_sites[i] for i in action]
                            rot_arrows = [arrow_sites[i] for i in action]
                            perm_arrows = [permutation[site] for site in rot_arrows
                                           if site >= 0]
                            if orig_sites == perm_sites and perm_arrows == arrow_config:
                                unique = False
                                break
                if accept is None or random() < accept:
                    if unique == True:
                        arsurvivors.append(coloring_with_arrows)
                        if verbosity is not None and verbosity >= 1 and not nested: #pragma: no cover
                            pbar.update(1)
                        if num_wanted is not None and len(arsurvivors) == num_wanted:
                            break
            orighash += 1
    else:
        from random import randrange
        visited = []
        orighash = randrange(maxpossible)
        while len(arsurvivors) < num_wanted:
            while orighash in visited:
                orighash = randrange(maxpossible)

            visited.append(orighash)
            
            arrow_config = _ainvhash(orighash,narrows,dim)
            unique = True
            temp_coloring_with_arroms = [0]*len(col)
            #we need to cycle through each of the operations in the
            #permutation group that got passed in to see if they will turn
            #the arrow configuration into one we've already seen.
            for perm in agroup:
                l = 0
                #this loop constucts the coloring with the correct arrows
                #for the permutation group to be applied to.
                for z in range(len(col)):
                    if col[z][0] < 0:
                        temp_coloring_with_arroms[z] = list(col[z])
                    elif col[z][0] >= 0:
                        temp_coloring_with_arroms[z] = list(col[z])
                        temp_coloring_with_arroms[z][0] = arrow_config[l]
                        l += 1

                new_arrow_config=[]
                arrow_perm = perm[1]
                color_perm = perm[0]
                new_coloring_with_arrows = []
                #here we use the permutations of the colors to permute the
                #colors for our configuration.
                new_coloring_with_arrows.extend([temp_coloring_with_arroms[color_perm[j]]
                                                 for j in range(len(temp_coloring_with_arroms))])
                #if there is an arrow on any lattice site it needs to
                #be updated. Otherwise just permute the colors.
                new_arrow_config.extend([arrow_perm[site[0]] for site in new_coloring_with_arrows
                                         if site[0] >= 0])
                #if the new configuration has a smaller hash number than
                #the current configuration then we have seen it before and
                #it is not unique.
                permhash = _ahash(new_arrow_config,dim)
                if permhash in visited and permhash != orighash:
                    unique = False
                    break
                #if none of the permutations have had a smaller hash number
                #than the current configuration then the configuration is
                #unique and we need to add it to the list.
            if unique == True:
                #this loop make the coloring with the arrows that will be
                #added to the list of unique configurations.
                coloring_with_arrows = [0]*len(col)
                i = 0
                for z in range(len(col)):
                    coloring_with_arrows[z] = list(col[z])
                    if col[z][0] >= 0:
                        coloring_with_arrows[z][0] = arrow_config[i]
                        i += 1

                # We need to check the superperiodic structures.
                if not supers:
                    for trans in translations:
                        action = trans[0]
                        permutation = trans[1]
                        orig_sites = [i[1] for i in coloring_with_arrows]
                        perm_sites = [orig_sites[i] for i in action]
                        perm_arrows = [permutation[site] for site in arrow_config
                                         if site >= 0]
                        if orig_sites == perm_sites and perm_arrows == arrow_config:
                            if action != list(range(len(action))) and permutation != list(range(len(permutation))):
                                unique == False

                if unique == True:
                    arsurvivors.append(coloring_with_arrows)
                
                if verbosity is not None and verbosity >= 1 and not nested: #pragma: no cover
                    pbar.update(1)
                # if num_wanted is not None and len(arsurvivors) == num_wanted:
                #     break
                        
    if verbosity is not None and verbosity >= 1 and not nested: #pragma: no cover
        pbar.close()

    return(arsurvivors)

# arrow_concs is a method that returns the concentration string
# including the arrows
def arrow_concs(cList,aconcs):
    """Uses the concentrations of the atoms and the arrows to make a
    labeling for the system. It then sorts them to be in the correct
    order for the code.

    :arg cListr: an integer array the concentration of the colors 
    :arg aconcs: an array of floats of the fractional number of arrows
    for each color
    """

    aconcs = [int(cList[i]*aconcs[i]) for i in range(len(cList))]
    
    decoration_w_arrows = []
    labels = range(1,len(cList)+1)
    for i in range(len(aconcs)):
        na = aconcs[i]
        ns = cList[i]
        if ns >= na:
            while na > 0:
                decoration_w_arrows.append([1,labels[i]])
                na -= 1
                ns -= 1
            while ns > 0:
                decoration_w_arrows.append([-1,labels[i]])
                ns -= 1
        else:
            while ns > 0:
                decoration_w_arrows.append([-1,labels[i]])
                ns -= 1

    decoration_w_arrows = _col_sort(decoration_w_arrows)
    
    return(decoration_w_arrows)

def get_arrow_concs(params):
    """If the concentrations are being restricted then find the correct 
    arrow for each species included.

    :arg params: the lattice.in parameters.
    """
    a_concs = []
    if params["is_crestricted"]:
        for c in params["concs"]:
            if (len(c) == 4 and params["arrows"]):
                a_concs.append(c[3])
            else:
                a_concs.append(0)
    else:
        for i in range(params["nspecies"]):
            a_concs.append(0)
    return a_concs

def enum_sys(groupfile, concs, a_concs, num_wanted, HNF, params, supers, accept=None):
    """Enumerates a random subset of the unique structures that have the shape
    defined by the symmetry group and the specified concentration.

    :arg groupfile: path to the file containing the symmetry group.
    :arg concs: list of integer concentrations for each species.
    :arg a_concs: list of integer *arrow* concentrations for each species.
    :arg num_wanted: the number of structures to pick randomly from the enumerated
      list.
    :args HNF: the HNF for the system we're currently enumerating
    :args params: the dictionary of parameters read in from lattice.in
    :args supers: True if superperiodic structures are to be kept.
    """

    from phenum.grouptheory import get_full_HNF, get_sym_group
    from phenum.polyaburnside import polya
    from phenum.tree import brancher, guess_and_check_brancher
    import phenum.io_utils as io

    cellsize = sum(concs)/len(params["basis_vecs"])
    decorations = arrow_concs(concs, a_concs)
    # get the symmetry group for this HNF. Assumes the group can be
    # found in the file labeled by (this_HNF)_sym_group.out
    if groupfile == None:
        sym_g = get_sym_group(params["lat_vecs"],params["basis_vecs"],get_full_HNF(HNF),3)
        agroup = []
        for i in range(len(sym_g.perm.site_perm)):
            agroup.append([sym_g.perm.site_perm[i],sym_g.perm.arrow_perm[i]])
    else:
        group = io.read_group(groupfile)
        # get symgroup from HNF and lat_vecs
        # add [0] to each element of the symmetry group
        agroup = [[g,[0]] for g in group]

    # we need to know the concentrations of the species with and
    # without arrows, we also need to know the number of arrows and
    # their species so we can undo the previous step later
    (n_arrows, arrow_types, sorted_concs) = how_many_arrows(decorations)

    # if we're enumerating a relatively large system (n>=10) but only
    # want a relatively small number of unique configurations (n<=100)
    # then we don't need to run the polya algorithm.
    if sum(concs) >=10 and num_wanted <= 100:
        total = 1e10
    else:
        # now find the number of unique arrangements using
        # polya
        if arrow_types != 0:
            # Since enumlib doesn't write the arrow group out we have to
            # recompute the group actions paired with their effects on the
            # arrows
            total = polya(sorted_concs, agroup, arrowings=arrow_types)
        else:
            total = polya(concs, agroup)
    
        # generate the random subset to be used. WARNING! for phonon enumerations
        #we have seen values that are *50* digits long! If the number exceeds
        #1e9, we change the approach to randomization.
        from phenum.msg import warn, err
        if num_wanted < total:
            if total < 1e6:
                from random import shuffle
                subset = list(range(1, total+1)) 
                shuffle(subset)
                subset = subset[0:num_wanted]
            elif accept is not None and (not isinstance(accept, float) or accept > 1.0):
                err("'-acceptrate' is to be a float less than 1.")
                exit(0)
            else:
                subset = num_wanted
        elif num_wanted == total:
            subset = []
        else:
            warn("number of configurations requested exceeds the number of "
                 "unique configurations available.")
            subset = []

    n_stabs = []
    # if we're doing a purely arrow enumeration then we don't need to
    # do the tree search but instead perform the final step of the
    # algorithm to find the possible unique displacements of the atoms
    from random import random
    if len(concs) == 1 and all(decorations) >=0:
        configs = []
        if float(num_wanted)/total < 0.001 and accept is None:
            small = True
            a_configs = add_arrows(decorations, agroup, 6, accept, num_wanted=num_wanted,small=True)
        else:
            a_configs = add_arrows(decorations, agroup, 6, accept, num_wanted=num_wanted)
            
        count = 1
        for config in a_configs:
            if isinstance(subset, list) and count in subset:
                configs.append(config)
            elif accept is not None or small:
                configs.append(config)
            count += 1
    else:
        if sum(concs) >= 10 and num_wanted <= 100:
            configs = guess_and_check_brancher(sorted_concs, agroup, decorations, 6, supers, cellsize, num_wanted)
        else:
            configs = brancher(sorted_concs, agroup, decorations, 6, supers, cellsize, total, subset, accept)

    if len(configs) != num_wanted and not super:              
        raise ValueError("Warning the enumeration code returned {} structures when {} "
                         "were asked for. This should not happen. Please submit a bug "
                         "report on https://github.com/wsmorgan/phonon-enumeration "
                         "including your input files so that this error may be "
                         "corrected.".format(str(len(configs)),str(num_wanted)))
        if len(configs) == 0:
            exit()

    return configs

def _ahash(coloring,dim):
    """Produces a unique number for each possible configuration of
    arrows. This number is used to compare the order that the arrows
    occure in.

    :arg coloring: is a 2D integer array of the full coloring with colors and arrows	    
    :arg dim: the number of directions the arrows can point
    """
    return sum([coloring[i]*dim**i for i in range(len(coloring))])

#anum is a unique number that is associated with an array of arrows.
#num_of_arrows in the number of arrows that are in the array.
#dim is the number of directions the arrows can point.
def _ainvhash(anum,num_of_arrows,dim):
    """Turns an arrow hash back into the array of arrow directions.

    :arg anum: the arrow hash number
    :arg num_of_arrows: the number of arrows in the system
    :arg dim: the number of directions the arrows can point
    """
    arrows = [0]*num_of_arrows
    for i in range(num_of_arrows):
        base = dim**(num_of_arrows-1-i)
        arrows[num_of_arrows-1-i] = anum//base
        anum -= base*arrows[num_of_arrows-1-i]
    return(arrows)



"""Methods for reading and writing enumeration and polya-counting results."""

"""Generates a random subset of possible structures weighted by the
Polya distribution for superstructures.
"""
def enum_data(cellsize, args, params): #pragma: no cover
    """Returns a list of dictionaries with the HNF, SNF, Left Transform
    and permutation group file names for the given cell size.
    """
    from os import path
    from numpy import loadtxt, array
    dirname = "{}{}".format(args["cellsdir"],args["dataformat"].format(cellsize))
    result = []

    if path.isdir(dirname):
        matrices = loadtxt(path.join(dirname, "matrices"), int)
        limit = matrices.shape[0] if len(matrices.shape) > 1 else 1
        matrices = array([matrices]) if limit == 1 else matrices
        for i in range(limit):
            result.append({
                "SNF": matrices[i,1:4],
                "HNF": matrices[i,4:10],
                "L": matrices[i,10:],
                "group": path.join(dirname, "group.{}".format(matrices[i,0]))})
    else:
        from phenum.msg import warn, err
        from os import system
        from phenum.io_utils import write_struct_enum, which
        warn("No data for cell size {0:d} found at {1}. Creating new {2} files now using "
             "{3}.".format(cellsize, dirname,args["dataformat"],args["exec"]))
        if which(args["exec"]) != None:
            write_struct_enum(params)
            system(args["exec"])
            system('rm symops_enum_parent_lattice.out readcheck_enum.out fort.*')
            if path.isdir(dirname):
                matrices = loadtxt(path.join(dirname, "matrices"), int)
                limit = matrices.shape[0] if len(matrices.shape) > 1 else 1
                matrices = array([matrices]) if limit == 1 else matrices
                for i in range(limit):
                    result.append({
                        "SNF": matrices[i,1:4],
                        "HNF": matrices[i,4:10],
                        "L": matrices[i,10:],
                        "group": path.join(dirname, "group.{}".format(matrices[i,0]))})
            else:
                err("The executable {0} does not produce the needed input folders {1}, "
                    "or else your struct_enum.in doesn't agree with your {2} file. "
                    "\nPlease ensure that your struct_enum.in is correct. If it is then "
                    "please follow the instructions found to compile {0} in the "
                    "README.".format(args["exec"],args["dataformat"],args["lattice"]))
                exit()                            
        else:
            err("Could not find {} on your path. Please add it to your path "
                "if you have already\n compiled it. Otherwise please follow the "
                "instructions found in the README to download, make,\n and place the "
                "executable in your path.".format(args["exec"]))
            exit()
        
    return result   

def _read_struct_enum(): #pragma: no cover
    """Reads in the struct_enum.out file and returns a dictionary with key
    for each cell size, and value a dictionary keyed by HNF with the number
    of unique structures.
    """
    i = 0
    structs = {}
    with open("struct_enum.out") as f:
        for line in f:
            i += 1                        
            if i > 17:
                vals = line.split()
                sN = int(vals[6])
                HNF = tuple(map(int, vals[11:17]))
                concs = tuple([vals[-1].count(c) for c in ["0", "1"]])
                if sN not in structs:
                    structs[sN] = {}
                if HNF not in structs[sN]:
                    structs[sN][HNF] = {}
                    structs[sN][HNF][concs] = 1
                else:
                    if concs in structs[sN][HNF]:
                        structs[sN][HNF][concs] += 1
                    else:
                        structs[sN][HNF][concs] = 1
    return structs

def _write_struct_summary(structs): # pragma: no cover
    """Writes the summary of unique structure counts by HNF and cell size
    to file for verification of polya.
    """
    from numpy import zeros
    for sN, HNFS in list(structs.items()):
        out = open('enum_'+str(sN)+'.out', 'w+')
        out.write('{0: <28}'.format("# HNF"))
        first = sorted(next(iter(HNFS.values())))
        for conc in first:
            out.write("{0: <10}".format(':'.join(map(str, conc))))
        out.write('{0: <10}\n'.format("Total"))

        conc_totals = {conc: 0 for conc in first}
        sHNFs = sorted(HNFS.values)
        for HNF in sHNFs:
            out.write("  {0: <26}".format(' '.join(map(str, HNF))))
            for conc in first:
                if conc in dHNF:
                    out.write("{0: <10d}".format(dHNF[conc]))
                    conc_totals[conc] += dHNF[conc]
                else:
                    out.write("{0: <10d}".format(0))
            out.write('{0: <10d}\n'.format(sum(dHNF.values())))
            
        out.write("# " + ''.join(['-' for i in range(len(first)*10 + 10 + 30)]) + '\n')
        out.write("{0: <28}".format(""))
        for conc in first:
            out.write("{0: <10d}".format(conc_totals[conc]))
        out.write("{0: <10d}\n".format(sum(conc_totals.values())))
        out.close()

def _distribute(cellsizes, ftype, n=None, dataformat="cells.{}",seed=None, res_type=None, res_values=None):
    """Returns a dictionary specifying how many of each cell shape, size and concentration
    to enumerate in order to obtain a grand total of 'n' unique cells, distributed according
    to the abundance of unique structures predicted by Polya. See the calling signature for
    method:distribution_fxn(). However, while that method does not guarantee the total 'n',
    this one does; if it is short a few at the end, it tops up the list according to the
    abundances across all cell sizes, shapes, etc.

    :arg cellsizes: a list of the cell sizes to include in the data set summary.
    :arg ftype: one of ['shape', 'conc', 'size', 'all']; determines the number of 
      arguments that the distr. function will accept and average over, thus controlling
      the granularity of the predictions.
    :arg n: the number of structure types to return, weighted by their polya distribution.
      If this value is None, then *all* structure types are returned with their unique
      number.
    :arg dataformat: the name of the directories to search for 'polya.out' files in.
    :arg seed: The seed for the random number generator.
    """
    (f, dataset, gtotal) = _distribution(ftype, None, None, cellsizes=cellsizes, dataformat=dataformat,
                                         res_values=res_values,res_type=res_type)
    #If they didn't specify an 'n', then we also return all the structures.
    if n is None:
        n = gtotal
    elif n > gtotal:
        from .msg import warn
        warn("The number of unique structures you requested ({}) ".format(n) +
             "exceeds the total number of unique structures ({}). ".format(gtotal) +
             "The code will return *all* structures for the distribution.")
        n = gtotal

    result = {}
    rtotal = [0]
    def vinsert(key, value, rtotal, n, result):
        if rtotal[0] < n: #Only update if we are deficient in structures.
            if value > 0:
                if key in result:
                    result[key] += value
                else:
                    result[key] = value
                rtotal[0] += value
            return True
        else:
            return False   
    
    def assign(relvals, f, rtotal, gtotal, n, result, ftype,seed_val=None):
        """Recurses through the relative weights specified to fill the
        result dictionary until it has the exact number of structures
        requested.
        """
        if n == gtotal:
            for key, value, limit in relvals:
                vinsert(key, limit, rtotal, n, result)
        else:
            from numpy import round
            from math import modf
            from random import random, seed, getstate
            from operator import itemgetter
            import sys
            if seed_val != None:
                seed(a=seed_val)
                
            ids = {
                "all": lambda key: key,
                "shape": lambda key: key[0:2],
                "conc": lambda key: (key[0], key[2]),
                "size": lambda key: (key[0],)
            }
            #Sort the relative values according to their weights so that
            #the most abundant ones pick up the extras.
            relvals = sorted(relvals, key=itemgetter(1), reverse=True)
            recount = 0
            while rtotal[0] < n and recount < 4:
                delta = n-rtotal[0]
                for key, value, limit in relvals:
                    #If the abundance guarantees an integer part, use that, but then
                    #use a random number to decide who to keep for fractional parts.
                    #Eventually this should terminate unless all of the fractional
                    #pieces are super small; but even then, we should get what we want
                    #by the time we have iterated through the whole list.
                    rf, qf = modf(value if recount == 0 else f(delta, *(ids[ftype](key))))
                    retval = int(qf + (1 if random() < rf else 0))
                    if not vinsert(key, retval, rtotal, n, result):
                        break
                recount += 1

            if recount == 3 and rtotal[0] < n: #pragma: no cover
                from .msg import warn
                warn("Reached maximum recursion limit in random assignment.")
                
    #First, compile a dictionary at the lowest level that has the weights
    #for the desired value of 'n'.
    relvals = []
    if ftype=="all":
        for size, data in list(dataset.items()):
            for HNF, distr in list(data["distr"].items()):
                for conc, limit in list(distr.items()):
                    key = (size, HNF, conc)
                    value = f(n, size, HNF, conc)
                    relvals.append((key, value, limit))
    elif ftype == "shape":
        for size, data in list(dataset.items()):
            for HNF, limit in list(data["stotals"].items()):
                key = (size, HNF, None)
                value = f(n, size, HNF)
                relvals.append((key, value, limit))                
    elif ftype == "conc":
        for size, data in list(dataset.items()):
            for conc, limit in list(data["ctotals"].items()):
                key = (size, None, conc)
                value = f(n, size, conc)
                relvals.append((key, value, limit))
    elif ftype == "size":
        for size, data in list(dataset.items()):
            limit = data["gtotal"]
            key = (size, None, None)
            value = f(n, size)
            relvals.append((key, value, limit))
    
    #This makes the selection according to the relative values and keeps
    #choosing, weighted by relative abundance, until we have the right
    #number of structures.
    assign(relvals, f, rtotal, gtotal, n, result, ftype,seed_val=seed)
    #Make sure we are returning exactly how many they asked for.
    if rtotal[0] > n: #pragma: no cover
        from .msg import warn
        warn("More structures were returned than asked for, should not be possible.")

    from operator import itemgetter
    return result

def _distribution(ftype, dataset, gtotal, cast=float, cellsizes=None, dataformat="cells.{}",res_type = None,
                  res_values=None):
    """Returns the Polya-weighted distribution function for the specified cell sizes.
    If dataset is None, a dataset and total are loaded using the specified cell sizes.
    This dataset and the total number of structures it contains are returned as
    arguments 2 and 3 of the tuple. Depending on the value of 'ftype', it returns 
    a tuple (f, dataset, gtotal) where 'f' is a function function with arguments:

    'all':   f(n ,size, HNF, conc)
    'shape': f(n, size, HNF)
    'conc':  f(n, size, conc)
    'size':  f(n, size)

    where 'size' is the integer cell size; 'HNF' is the array of 6-elements defining
    the HNF matrix that is the cell shape; 'conc' is a list of concentrations such as
    [1,5] for a 6-atom cell; 'n' is the *grand* total number of structures desired from
    calling the function over all possible values of the various parameters passed to
    the distribution function.

    The value returned by the function is an integer specifying how many of that
    structure type should be selected according to the total distribution across
    all structures included in cellsizes.

    :arg dataset: the dictionary returned by method:_distribution_summary().
    :arg gtotal: the total number of unique structures corresponding to the
      dictionary 'dataset'.
    :arg ftype: one of ['shape', 'conc', 'size', 'all']; determines the number of 
      arguments that the distr. function will accept and average over, thus controlling
      the granularity of the predictions.
    :arg cellsizes: a list of the cell sizes to include in the data set summary.
    :arg dataformat: the name of the directories to search for 'polya.out' files in.
    :arg res_type: 'shape' if shapes are being restricted, or 'conc' if concentrations are  being restricted.
    :arg res_values: The allowed values of the restricted parameter.
    """
    if dataset is None:
        if cellsizes is not None:
            if res_type is not None:
                if res_type == "shape":
                    (dataset, gtotal) = _distribution_summary(cellsizes, dataformat=dataformat, HNFs=res_values)
                elif res_type == "conc":
                    (dataset, gtotal) = _distribution_summary(cellsizes, dataformat=dataformat, wanted_concs=res_values)
                else:
                    raise ValueError("Cannot filter the distribution using {}. Please use 'shape' or 'conc'.".format(res_type))

            else:
                (dataset, gtotal) = _distribution_summary(cellsizes, dataformat=dataformat)
        else:
            from .msg import err
            err("No dataset or cell sizes specified.")
            return None
        
    #We just need to sum up the total number of unique structures across all the
    #given cell sizes and then create a function that weights a total number of
    #desired structures by concentration and cell shape.
    ftotal = float(gtotal)
    if ftype == "all":
        f = lambda n, size, HNF, conc: cast(dataset[size]["distr"][tuple(HNF)][conc]/ftotal*n)
    elif ftype == "shape":
        f = lambda n, size, HNF: cast(dataset[size]["stotals"][tuple(HNF)]/ftotal*n)
    elif ftype == "conc":
        f = lambda n, size, conc: cast(dataset[size]["ctotals"][tuple(conc)]/ftotal*n)
    elif ftype == "size":
        f = lambda n, size: cast(dataset[size]["gtotal"]/ftotal*n)
    else:
        raise ValueError("The parameter {} is not a valid parameter for the distribution. "
                         "Please use size, shape, conc, or all.".format(ftype))

    return (f, dataset, gtotal)
        
def _distribution_summary(cellsizes, HNFs = None, wanted_concs = None, dataformat="cells.{}"):
    """Returns a dictionary that summarizes the Polya predictions for unique structures
    by cell size, shape and concentration for use by a distribution function. Returns
    a tuple (summary dict, grand total), where grand total is the total number of structures
    predicted for *all* cell sizes.

    :arg cellsizes: a list of the cell sizes to include in the data set summary.
    :arg dataformat: the name of the directories to search for 'polya.out' files in.
    :arg HNFs: a list of the allowed HNFs.
    :arg wanted_concs: a list of the allowed concentrations.
    """
    from os import path
    from numpy import loadtxt
    gtotal = 0
    dataset = {}
    for s in cellsizes:
        dirname = dataformat.format(s)
        source = path.join(dirname, "polya.out")
        if not path.isfile(source):
            from .msg import err
            err("Cannot find polya distribution for size {} at {}".format(s, source))
            continue

        #First load the possible concentrations from the commented first line.
        with open(source) as f:
            headings = f.readline().split()
            concs = [tuple(map(int, h.split(":"))) for h in headings[2:-1]]
        polya = loadtxt(source, int)
        distr = {}

        if HNFs == None and wanted_concs == None:
            for iHNF, HNF in enumerate(polya[0:-1,0:6]):
                distr[tuple(HNF)] = {tuple(c): v for c, v in zip(concs, polya[iHNF,6:-1])}
            stotals = {tuple(m): v for m, v in zip(polya[0:-1,0:6], polya[:-1,-1])}
            ctotals = {c: v for c, v in zip(concs, polya[-1, 6:-1])}
        elif HNFs != None:
            ctotals = {c: v for c, v in zip(concs,[0]*len(concs))}
            stotals = {}
            for iHNF, HNF in enumerate(polya[0:-1,0:6]):
                if tuple(HNF) in HNFs:
                    distr[tuple(HNF)] = {tuple(c): v for c, v in zip(concs, polya[iHNF,6:-1])}
                    for c, v in zip(concs, polya[iHNF,6:-1]):
                        ctotals[c] += v
            for m, v in zip(polya[0:-1,0:6], polya[:-1,-1]):
                if tuple(m) in HNFs:
                    stotals[tuple(m)] = v
        elif wanted_concs != None:
            ctotals = {c: v for c, v in zip(concs,[0]*len(concs))}
            stotals = {}
            for iHNF, HNF in enumerate(polya[0:-1,0:6]):
                cs = []
                vs = []
                count = 0

                for c, v in zip(concs, polya[iHNF,6:-1]):
                    if c in wanted_concs:
                        ctotals[c] += v
                        cs.append(c)
                        vs.append(v)
                        count += 1

                distr[tuple(HNF)] = {tuple(c): v for c, v in zip(cs, vs)}
            for m, v in zip(polya[0:-1,0:6], polya[:-1,-1]):
                if tuple(m) in distr:
                    stotals[tuple(m)] = v

        dataset[s] = {
            "distr": distr,
            "concs": concs,
            "ctotals": ctotals,
            "stotals": stotals,
            "gtotal": sum([ctotals[key] for key in ctotals])
        }
        gtotal += dataset[s]["gtotal"]

    return (dataset, gtotal)    

def _print_distribution(distr, distribution, filename=None, header=True, append=False, show=False):
    """Prints the specified distribution to screen or file.

    :arg distr: the distribution returned by method:distribute().
    :arg distribution: The type of distribution, i.e., 'shape', 'size', 'conc', or 'all'.
    :arg filename: The output file name.
    :arg header: True if the header is to be included in the file.
    :arg append: True if the file is to be appended to.
    :arg show: True if the distribution is to be printed to the screen.
    """
    if show:
        from .msg import arb, cenum
        bysize = {}
        sfmt = " {0: <5d} | {1: <15} | {2: <5} | {3: <5d} "
        for key, value in list(distr.items()):
            size, HNF, conc = key
            skey = sfmt.format(size, "" if HNF is None else ' '.join(map(str, HNF)),
                               "" if conc is None else ':'.join(map(str, conc)), value)
            cols = (cenum["cwarn"], cenum["cinfo"], cenum["cgens"], cenum["cokay"])
            if (size, value) in bysize:
                bysize[(size, value)].append((skey, cols, tuple(HNF) if HNF is not None else None))
            else:
                bysize[(size, value)] = [(skey, cols, tuple(HNF) if HNF is not None else None)]

        from operator import itemgetter
        for size, value in sorted(bysize.keys()):
            for skey, cols, HNF in sorted([[(i or "") for i in x] for x in bysize[(size, value)]], key=itemgetter(2)):
                arb(skey, cols, "|")

    if filename is not None:
        #We don't worry about ordering it, just write them to file in whatever
        #order the keys are in.
        with open(filename, 'w' if not append else 'a') as f:
            if header:
                if distribution == "all":
                    f.write("# {0: <28}  {1: <10}  {2}\n".format("HNF", "Conc.", "Number"))
                elif distribution == "shape":
                    f.write("# {0: <18}  {1}\n".format("HNF", "Number"))
                elif distribution == "conc":
                    f.write("# {0: <6}  {1: <6}  {2}\n".format("Size", "Conc.", "Number"))
                else:
                    f.write("# {0: <6}  {1}\n".format("Size", "Number"))
            for key, value in list(distr.items()):
                size, HNF, conc = key
                if conc == None:
                    conc = []
                if distribution == "all":
                    f.write("  {0: <28}  {1: <10}  {2:d}\n".format(' '.join(map(str, HNF)), ' '.join(map(str, conc)), value))
                elif distribution == "shape":
                    f.write("  {0: <18}  {1}\n".format(' '.join(map(str, HNF)), value))
                elif distribution == "conc":
                    f.write("  {0: <6}  {1: <6}  {2:d}\n".format(size, ' '.join(map(str, conc)), value))
                else:
                    f.write("  {0: <6}  {1:d}\n".format(size, value))
                    
def make_enum_in(distribution,directory,outfile,number=None,dataformat="cells.{}",sizes=None,save=True,seed=None,
                 restrict=None):
    """Makes an enum.in file if the distrubiton type is all with the
    desired number of structures. Otherwise prints the distribution
    information to the screen for the user.

    :arg distribution: The parameters that the distribution is over
    ('shape', 'conc', 'size', 'all').
    :arg n: The number of structures or 'all'.
    :arg dataformat: The folder name for the cell sizes.
    :arg sizes: when specified, limit the distribution to these integer cell sizes;
      otherwise, look for all cell sizes we have data for.
    :arg directory: The directory that contains the folders with the cell sizes.
    :arg outfile: The name of the output file for the distribution
    :arg save: True if the data is to be saved to file.
    :arg seed: The seed for the random number generator.
    :arg restrict: A list containing the restriction type ('shape','conc') and the file of allowed values.
    """

    from os import listdir, chdir, getcwd

    initial_directory = getcwd()
    if directory != "":
        chdir(directory)
    
    files = listdir(".")
    if sizes is None:
        sizes = []
        form = dataformat.split(".")[0]

        ready = False
        for f in files:
            if form in f:
                sizes.append(int(f.split(".")[1]))
                ready = True
    else:
        for i in sizes:
            celldir = dataformat.format(i)
            if celldir not in files:
                raise ValueError("Cannot find '{}' in current directory.".format(celldir))
        ready = True
                
    if ready == False:
        raise ValueError("The files {} don't exist in this directory. You either need to run"
            " the -polya option or else navigate into the folder that contains "
            "the output {} folders.".format(dataformat))

    if restrict is not None:
        # from numpy import loadtxt
        res_values = []
        with open(restrict[1],"r") as resf:
            for values in resf:
                if "#" in values:
                    pass
                elif restrict[0] == "shape":
                    res_values.append(tuple([int(i) for i in values.split()[0:6]]))
                elif restrict[0] == "conc":
                    res_values.append(tuple([int(i) for i in values.split()[1:3]]))

        distr = _distribute(sizes,distribution,n=number,dataformat=dataformat,seed=seed,res_type=restrict[0],res_values=res_values)        
    else:
        distr = _distribute(sizes,distribution,n=number,dataformat=dataformat,seed=seed)

    if initial_directory != getcwd():
        chdir(initial_directory)
        
    if distribution.lower() == "all":
        _print_distribution(distr,distribution,filename=outfile)
    else:
        _print_distribution(distr,distribution,filename= outfile if save else None,show=True)



"""The heart of the code is here. These methods build the tree and
check the canfigurations for uniqueness."""

def _coefficients(number_of_each_color,number_of_slots):
    """This method generates the radix numbers, or total number of
    combinations of each color, for a given aray.

    :arg number_of_each_color: is an integer array integer indicating the
    number of times each color is found in the array.
    :arg number_of_slots: an integer equal to the sum of the
    number_of_each_color array.
    """
    from .numerics import binomial_coefficient

    #renames variables for convience and creates an empty array
    coe = []
    for i in range(0,len(number_of_each_color)):
    #Uses the binomial_coefficient method to build an array of radix numebrs
        coe.append(binomial_coefficient(number_of_slots,number_of_each_color[i]))
        number_of_slots -= number_of_each_color[i]
    return coe

def _hash(listi,color):
    """Generates a hash for each color in the ladeling returning a array
    of the hash for each color.
    
    :arg listi: An integer array that contains the labeling.
    :arg color: An integer array of the concentrations of the
    colors.

    For details on this method see: http://msg.byu.edu/papers/enum3.pdf
    """
    from .numerics import binomial_coefficient

    m = len(listi)
    li = listi[::-1]
    rm1 = m-li.index(1)
    y = 0
    z = listi[:rm1].count(0)#m - listi[:rm1].count(1)
    li = listi
    for i in range(z):
        p0 = li.index(0) #next(x[0] for x in enumerate(li) if x[1] > j)
        k = li[p0:].count(1)
        li = li[p0+1:]
        y += binomial_coefficient(m-(p0+1),k-1)
        m -= (p0 + 1)

    return y

def _invhash(branch,colors,n):
    """Turns a hash array into a labeling, i.e., undoes the hash()
    subroutine.
    
    :arg branch: an integer list of the radix number/hash array
    :arg colors: an integer array of the concentration of the colors
    :arg n: the length of the labeling array, i.e., the
    number of sites in the system

    For the details on this method see: http://msg.byu.edu/papers/enum3.pdf
    """

    from .numerics import binomial_coefficient

    new = [0 for y in range(n)]
    coluse = 1
    for i in range(0,len(branch)):
        newi = [0 for y in range(n)]
        I = branch[i]
        t = colors[i]
        for l in range(n,0,-1):
            binom = binomial_coefficient(l-1, t-1)
            if binom <= I:
                I -= binom
            else:
                newi[n-l] = coluse
                t -= 1
        coluse += 1
        n -= colors[i]
        si = 0
        for s in range(0,len(new)):
            if new[s] == 0:
                new[s] = newi[si]
                si += 1
    return(new)

#color_list takes the configuration and returns a unique list of the
#colors used without duplicates.
def _color_list(col):
    """Finds the unique colors in a labeling.

    :arg col: an integer array of the labeling.
    """
    colors=[]

    tcol=col[:]
    for i in range(len(tcol)):
        if tcol[i] != 0:
            colors.append(tcol[i])
            for j in range(len(tcol)):
                if tcol[j] == tcol[i] and i != j:
                    tcol[j] =0
            tcol[i] =0
    return(colors)



def _perm(casei,colors,length,index,gen,stab,order,ast):
    """This method applies a cyclical permutation to an array to determine
    if it is unique. 
		
    :arg casei: the integer array hash for the current branch of
    the tree that will be permuted
    :arg colors: an integer array indicates the concentration of
    each color in the system
    :arg ast: the stabalizer for the final configuration that will
    be passed to the arrow permutation routine.
    :arg length: the total number of sites (colors) in the system
    :arg index: an integer that indicates which level of the tree
    we are in, i.e., which color is being added to the tree
    :arg gen: a 3D integer array that contains the permutation group
    :arg stab: a 3D integer array of the stabilizers for
    the previous level of this branch in the tree
    :arg order: is an integer array that keeps track of which
    configurations of the first level of the tree have already
    been seen

    The routin outputs the following:

    unique: an integer that indicates if the configuration is
    unique, a returned value of 0 is a unique configuration
    anything else is not
    st: a 2D integer array that stores the stabilizers for this
    configuration		   
    order and ast
    """
		
    #initialize unique to be 0 and arrays to be empty
    unique = 0
    if stab == 0:
        stab = []
    st = []
    ast = []
		
    # uses the invhash method from Inverse_radix_num to turn the id
    # number, casei, into an array
    li = list(_invhash(casei,colors,length))

    # if we're on the first level of the tree then we need to
    # follow a special procedure
    if index == 0:
	# first apply all the symmetry operations one at a time, a
	# is a single operation
        for a in gen:
	    # get the effect of the group on the colors, ingore the arrows
	    # i is the permutation of the colors for this operation
            i = a[0]
            lnew = []
	    # apply the permutation to get an equivalent configuration
            for j in i:
                lnew.append(li[j])
            lnewb = list(lnew)
                
	    # any color that was placed that isn't the color for the
	    # current level is set to zero
            for y in range(0,len(lnew)):
                if lnew[y] != index + 1:
                    lnewb[y] = 0
                elif lnew[y] == index + 1:
                    lnewb[y] = 1

	    # hash from radix_num_generator turns the array back into
	    # an integer hash array
            rtest = _hash(list(lnewb),colors)

	    # if the original hash is smaller than the new one we
	    # don't want to visit the new one again so we set its
	    # location in order to be -1
            if casei[0] < rtest:
                order[rtest] = -1
            
	    # if the original hash is larger than the new one then
	    # this configuration isn't unique so we set unique to be 1
	    # and reset the other variables
            elif casei[0] > rtest:
                unique += 1
                st = []
                ast = []
                break

	    # if the symmetry group didn't change this level
            # of the hash then it is a stabilizer and needs to
            # be saved
            elif casei[0] == rtest:
                st.append(a)

	    # if the symmetry op changed nothing in the
	    # configuration then it is a stabilizer for the
	    # arrow level and needs to be saved
            if li == lnew:
                ast.append(a)

    # if there is only a single stabilizer then the configuration is
    # unique and the stabilizer needs to be used for any later braches
    elif len(stab) == 1:
        st = stab
        ast = stab

    # if there are multiple stabilizers then we need to apply each of
    # them just like we needed to apply the entire group on the first
    # level of the tree
    elif len(stab) > 1:
        ast = []
        # first apply all the stabilizers one at a time, a is a single
        # operation
        for a in stab:
	    # get the effect of the group on the colors, ingore the
	    # arrows i is the permutation of the colors for this
	    # operation
            i = a[0]
            lnew = []
	    # apply the permutation to get an equivalent configuration
            for j in i:
                lnew.append(li[j])
                lib= list(li)
            lnewb = list(lnew)
            lnewb2 = []
	    # any color that was placed that isn't the color for
	    # the current level is set to zero
            for y in range(0,len(li)):
                if li[y] > index + 1:
                    lib[y] = 0
            for y in range(0,len(lnew)):
                if lnew[y] > index + 1:
                    lnewb[y] = 0
            for y in range(0,len(lnew)):
                if lnewb[y] == 0:
                    lnewb2.append(0)
                elif lnewb[y] == index + 1:
                    lnewb2.append(1)
                    
	    # if the the new and original configurations are the
	    # same then save the stabilizers
            if lnewb == lib:
                st.append(a)
            if li == lnew:
                ast.append(a)

	    # hash from radix_num_generator turns the array back
	    # into an integer hash array
            rtest = _hash(list(lnewb2),colors)
	    # if the original hash is larger than the new one then
	    # this configuration isn't unique so we set unique to
	    # be 1 and reset the other variables
            if casei[index] > rtest:
                unique = 1
                st = []
                ast = []
                break
            
	    # if the configuration isn't unique break from the
            # loop
            if unique == 1:
                ast = []
                st = []
                break
    else:
        unique = 1
		
    return(unique,st,order,ast)


def brancher(concs,group,colors_w_arrows, dim, supers, cellsize, total=0, subset=None, accept=None):
    """This routine navigates the tree and saves the unique configurations
    to an array survivors.
		
    :arg concs: the concentrations of the colors or atoms
    :arg group: the symmetry group for the system, including the
    effect on the arrows (displacement directions)
    :arg colors_w_arrows: an integer 2D array that indiciates
    which atoms are being displaced, i.e., where the arrows are.
    :arg dim: the number of directions the arrows can point
    :arg total: the total number predicted by polya.
    :arg subset: an integer array of the subset of unique
    arrangements wanted
    :arg accept: for large enumerations, how often to accept configurations.
    :arg supers: Logical that indicates if super periodic structures are to be kept.
    :arg cellsize: The number of cells in the system 

    The method returns the list of unique configurations and the
    number of stabilizers for the last level of the tree.

    survivors: is the 3D array of the unique arrangements of
    colors and arrows
    stob_len: the number of stabilizers for a given unique
    configuration
    """

    from .phonons import how_many_arrows, add_arrows
    from copy import deepcopy

    # initial setup

    # redifine the colors so that the arrows are treated like
    # their own color
    colors = _color_list(colors_w_arrows)
    # find the total number of atoms
    n = sum(concs)

    # Find the mixed radix number counter for the system
    C = _coefficients(concs,n)

    # count how many arrows there are
    (narrows,arrow_types,concs_w_arrows) = how_many_arrows(colors_w_arrows)
    # prepare the stabalizer array to be the appropriate size
    stabalizer = [0]*(len(concs)+1)
    
    # ast is the stabalizer for the arrow permutaiton order is used to
    # track which branchs of the first layer we have already seen so
    # we can skip over the ones we already know aren't unique
    ast = []
    order = {}
    for i in range(0,C[0]+1):
        order[i] = i

    # determine if we are finding a subset of doing a full enumeration
    if subset is not None and isinstance(subset, list) and len(subset) > 0:
        use_subset = True
    elif (subset is None or isinstance(subset, int)) and accept is not None:
        use_subset = True
    else:
        use_subset = False
				
    # count is an integer counter used to keep track of where in
    # the total number of configurations we are.
    count = 1

    #create the all zero branch of lables
    #survivors is an array that stores the unique configurations
    from numpy import zeros
    branch = zeros(len(C))
    survivors = []

    #variables used for iteration and as test criteria for
    #navigating the tree
    # i keeps track of which level of the tree we are on
    # b0 lets us know when we've used all the needed colors/been
    # to all the relevant levels
    i = 0
    b0 = 0

    from random import random
    from phenum.msg import verbosity
    if verbosity is not None and verbosity >= 1: #pragma: no cover
        from tqdm import tqdm
        if isinstance(subset, list) and len(subset) > 0:
            ntotal = len(subset)
        elif isinstance(subset, int):
            ntotal = subset
        else:
            ntotal = total
        pbar = tqdm(total=ntotal)
    # Now we loop through the different possible hash arrays until
    # they have all been considered
    ncurrent = 0
    while branch[0] < C[0] and ((subset is not None and isinstance(subset, list) and len(survivors) < len(subset))
                                or use_subset == False
                                or (accept is not None and isinstance(subset, int) and len(survivors) == subset)):
	# perm determines if the new array is unique, if yes unique =
	# 0, if no then unique = 1, perm also outputs the stabilizers
	# for each level, the order for the first level, and the
	# stabilizers for the arrow configurations
        (unique,stabalizer[i+1],order,ast) = _perm(branch,concs,n,i,group,stabalizer[i],order,ast)

        if not supers and narrows == 0:
            from operator import mul
            # if we have a unique structure then we need to check if
            # it's super periodic before saving it.
            if unique == 0 and (b0 == 1 or i == 0):
                brancht = list(_invhash(branch, concs, len(colors_w_arrows)))
                for trans in range(1,int(cellsize)):
                    action = group[trans][0]
                    trans_branch = []
                    for act in action:
                        trans_branch.append(brancht[act])
                    if trans_branch == brancht and action != list(range(cellsize)):
                        unique = 1
                        if use_subset:
                            if count in subset:
                                loc = subset.index(count)
                                nv = count
                                while nv in subset and nv < reduce(mul,C,1):
                                    nv += 1
                                subset[loc] = nv

                            count += 1
                        break
	# if this array is unique then we may need to append it to the
	# list of survivors
        if unique == 0:
	    # if we've visited every level of the tree
	    # then we need to append the unique
	    # configurations
            if b0 == 1 or i == 0:
		# if there are arrows we need to see
		# what their unique arrangements are
		# before saving the configurations
                if narrows > 0:
		    # first use invhash to turn the hash back to an array
                    brancht = list(_invhash(branch, concs, len(colors_w_arrows)))
		    # make a coloring with arrows to be passed to the
		    # arrow permutiation code by adding arrows back
		    # into the array where needed.
                    for z in range(len(brancht)):
                        brancht[z] = deepcopy(colors[brancht[z] -1])
		    # add_arrows from the phonon_brancher code returns
		    # the unique configurations with the unique arrow
		    # arrangements
                    arsurvivors = add_arrows(brancht,ast, dim, group[0:cellsize], accept, True,supers=supers)
		    #write the unique confgurations to file.
                    for z in arsurvivors:
			# if we aren't using a subset write everything
			# to file.
                        if not use_subset:
                            survivors.append(z)
			# if we are then we only want to write the
			# random subset to file.
                        else:
                            if subset is not None and isinstance(subset, list) and count in subset:
                                survivors.append(z)
                            elif accept is not None:
                                survivors.append(z)
                        count += 1
                else:
		    # if there are no arrows just
		    # write the unique arrangement
		    # to file
                    if not use_subset:
                        tbranch = list(_invhash(branch, concs, len(colors_w_arrows)))
                        tbranch = [colors[leaf -1][1] for leaf in tbranch]
                        survivors.append([[-1,leaf] for leaf in tbranch])
                    else:
                        if subset is not None and isinstance(subset, list) and count in subset:
                            tbranch = list(_invhash(branch, concs, len(colors_w_arrows)))
                            tbranch = [colors[leaf -1][1] for leaf in tbranch]
                            survivors.append([[-1,leaf] for leaf in tbranch])
                        elif accept is not None and random() < accept:
                            tbranch = list(_invhash(branch, concs, len(colors_w_arrows)))
                            tbranch = [colors[leaf -1][1] for leaf in tbranch]
                            survivors.append([[-1,leaf] for leaf in tbranch])
                    count += 1

		    # if we aren't on the last contributing level
		    # of the tree then make sure b0 is 0
            if i < len(branch) - 2:
                b0 = 0

	# all that follows dictates how we navigate the tree. This can
	# be a little tricky but does work at this time. Basically try
	# not to mess with the last few lines of this code if it can
	# be helped
								
	# if b0 is 0 then we need to move up to the next level of the
	# tree at this point
        if b0 == 0:
            if i < len(branch)-1:
                i += 1
	    # if we are now looking at the last level then set b0 to 1
	    # so that any unique configurations found will be saved
            if i >= len(branch) - 2:
                b0 = 1

	# root is a variable that tells us if we've filled this
	# level of the tree yet
        root = 0
	# if root = 0 and i>0 then we need to check to see if
	# we need to back track up the tree yet
        while root == 0 and i > 0:
	    # if we've used all the configurations for this level then
	    # we need to go back a level, if not then set root to 1 so
	    # that we keep moving forward through the tree
            if branch[i] == C[i]-1:
		# first reset this level's number so that it will
		# increment properly the next time it's visited
                branch[i] = 0  
		# next reduce i by 1 to go back a level
                i -= 1
		# if i = 0 we're back to the bottom of the tree. We
		# need to reset the stabilizers, b0 and increment the
		# bottom layers index, branch[i], by 1
                if i == 0:
                    stabalizer = [0]*len(concs)
                    b0 = 0
                    branch[i] += 1
            else:
                root = 1
                
	# if b0=1 then we need to increase this level index,
	# branch[i], by 1 to consider the next configuration
        if b0 == 1:
            branch[i] += 1

	# test helps us check the first level for areas we can skip
        test = 0
        while test == 0:
	    # if we're on the first branch and the order
	    # for this configuration is negative then we
	    # skip it. Otherwise we need to consider it.
            if i == 0 and order[branch[i]] == -1:
                branch[i] += 1
            else:
                test = 1
                
        if len(survivors) > ncurrent:
            ncurrent = len(survivors)
            if verbosity is not None and verbosity >= 1: #pragma: no cover
                pbar.update(1)
                
    if verbosity is not None and verbosity >= 1: #pragma: no cover
        pbar.close()
    # done
    return survivors

def guess_and_check_brancher(concs, group, colors_w_arrows, dim, supers, cellsize, num_wanted):
    """When the number of configurations wanted is sufficiently small
    relative to the total number of unique configurations then it is
    faster to 'pick' them from the list of possible configurations and
    then verify that none of your selected configurations are
    equivalent. 

    :arg concs: the concentrations of the colors or atoms
    :arg group: the symmetry group for the system, including the
    effect on the arrows (displacement directions)
    :arg colors_w_arrows: an integer 2D array that indiciates
    which atoms are being displaced, i.e., where the arrows are.
    :arg dim: the number of directions the arrows can point
    :arg supers: Logical that indicates if super periodic structures are to be kept.
    :arg cellsize: The number of cells in the system 

    The method returns the list of unique configurations.

    survivors: is the 3D array of the unique arrangements of
    colors and arrows
    """

    from .phonons import how_many_arrows, add_arrows
    from copy import deepcopy

    # initial setup

    # redifine the colors so that the arrows are treated like
    # their own color
    colors = _color_list(colors_w_arrows)
    # find the total number of atoms
    n = sum(concs)

    # Find the mixed radix number counter for the system
    C = _coefficients(concs,n)

    # count how many arrows there are
    (narrows,arrow_types,concs_w_arrows) = how_many_arrows(colors_w_arrows)
    # prepare the stabalizer array to be the appropriate size

    from random import random, randint
    from phenum.msg import verbosity
    if verbosity is not None and verbosity >= 1: #pragma: no cover
        from tqdm import tqdm
        pbar = tqdm(total=num_wanted)
    
    survivors = []
    visited = []

    nfound = 0
    while len(survivors) < num_wanted:
        print(len(survivors))
        candidate = []
        # pick a random configuration to check against.
        for i in range(len(C)):
            candidate.append(randint(0,C[i]))

        while candidate in visited:
            for i in range(len(C)):
                candidate.append(randint(0,C[i]))

        visited.append(candidate)
        unique = True

        config = list(_invhash(candidate,concs,sum(concs)))
        for perms in group:
            perm = perms[0]

            new_config = []
            for j in perm:
                new_config.append(config[j])
            new_config = list(new_config)

            new_location = _hash(new_config,concs)

            if new_location in visited and new_location != candidate:
                unique = False
                break
            
        # if there are no arrows and we don't want to store
        # superperiodic structures then we need to see if any
        # translations operations are stabilizers of the
        # configuration to eliminate the unwanted structures.
        if not supers and narrows == 0 and unique == True:
            for trans in range(1,cellsize):
                action = group[trans][0]
                trans_config = []
                for act in action:
                    trans_config.append(config[act])
                if trans_config == config and action != list(range(cellsize)):
                    unique = False
                    break
                
        if unique == True:
            if narrows > 0:
                # We need to make a copy of the branch with arrows on
                # it to pass to the add_arrows code.
                t_config = list(_invhash(candidate,concs,sum(concs)))

                for con in range(len(t_config)):
                    t_config[con] = deepcopy(colors[t_config[z] -1])
                    
                arsurvivors = add_arrows(t_config, group, dim, group[0:cellsize], nested=True, num_wanted = num_wanted, supers = supers, small = True)

                for survivor in arsurvivors:
                    survivors.append(survivors)
                    
            else:
                survivors.append([[-1,leaf] for leaf in config])
            
        if len(survivors) > nfound:
            nfound = len(survivors)
            if verbosity is not None and verbosity >= 1: #pragma: no cover
                pbar.update(1)
    if verbosity is not None and verbosity >= 1: #pragma: no cover
        pbar.close()
    # done
    return survivors
            



#!/usr/bin/env python
from phenum import msg

def RepresentsInt(s):
    """Determines if a stri could be an int.

    :arg s: The string to be tested.
    """    
    try: 
        int(s)
        return True
    except ValueError:
        return False

def _make_structures(args):
    """Makes a VASP POSCAR file for the desired structures."""
    from phenum.io_utils import read_enum_out, write_POSCAR
    from phenum.vector_utils import map_enumStr_to_real_space, cartesian2direct

    (system, structure_data) = read_enum_out(args)

    # for each structure write the vasp POSCAR
    for structure in structure_data:
        # space_data is a dictionary containing the spacial data for
        # the structure
        space_data = map_enumStr_to_real_space(system,structure,args["mink"])

        space_data["aBas"] = cartesian2direct(space_data["sLV"],
                                              space_data["aBas"],system["eps"])

        write_POSCAR(system,space_data,structure,args)
        

def examples():
    """Print some examples on how to use this python version of makeStr."""
    script = "makeStr: Makes a vasp style POSCAR for the desired system."
    explain = ("For all the examples bellow it is assumed you have already "
               "run the enumeration code and produced an enum.out style file.")
    contents = [("Make a single POSCAR file",
            "To make a POSCAR file for a specific structure listed in the "
            "`enum.out` style file you will need to identify the structure \n number "
            "(the first number of each row in the file) for the structure you want "
            ". For example to make a POSCAR for structure number 10 \n from an `enum.out` "
            "file.","makeStr.py 10 \n"),
           ("Make multilpe POSCARS at once",
            "To make multiple POSCARS for a range of values in the `enum.out` style "
            "file simply list the starting and ending structure numbers \n of the range. "
            "To make POSCARS for every structure in the output file use the word `all`.",
            "makeStr.py 10 20 \n  makeStr.py all \n"),
            ("Find the lattice parameter for the system",
             "To have makeStr.py predict the lattice parameter for the system using "
             "Vegard's Law use the -species option followed by a space \n separated list "
             "of the elements in the system.","makeStr.py 10 -species Al Cu \n"),
            ("Include displacements in POSCAR",
             "If `arrows` (displacement directions) were included in the enumeration "
             "then it is possible to displace them off the lattice points \n when making the "
             "POSCARS using the -displace option followed by the displacement amount "
             "expressed in terms of the lattice parameter. \n In other words if `a` is "
             "the lattice parameter and the atoms were to be displaced by `a/2` then "
             "the command would be:","makeStr.py 10 -displace 0.5 \n"),
            ("Make displacements have different lengths in POSCAR",
             "If `arrows` were included in the model and the `-displace` flag is being "
             "used it is possible to 'rattle' the displacements so that \n they are not all "
             "the same length. Using the `-rattle` option applies a random distribution "
             "to the displacements with the larges change \n in the displacements specified "
             "by the user as a fraction of the displacement given. So if a displacement of "
             "0.5 was given and the \n displacements were to be randomized by 1/4 of that total "
             "displacement the the command would be:",
             "makeStr.py 10 -displace 0.5 -rattle 0.25")]
    required = ("REQUIRED: A `enum.out` file.")
    output = ("RETURNS: A vasp style POSCAR labeled vasp.* where the `*` is replaced "
              "with the structure number for the `enum.out` file.")
    details = ("")
    outputfmt = ("")

    msg.example(script, explain, contents, required, output, outputfmt, details)

script_options = {
    "structures": dict(nargs="+",
                        help=("The desired structure numbers from the enum.out file. This "
                              "can be either a single value or a desired range indicated by "
                              "the starting and stopping structure numbers.")),
    "-displace": dict(default=0.0, type=float,
                        help=("The displacement amount for the arrows in units of the lattice "
                               "parameter. Default is 0.")),
    "-input": dict(default="struct_enum.out",type=str,
                        help=("Override the default 'struct_enum.out' file name.")),
    "-mink": dict(default="t", choices=["t","f"],
                        help=("Sets flag to perform minkowski reduction of the basis (T/F)."
                              " Default is True.")),
    "-species": dict(default=None, nargs="+",type=str,
                        help=("Specify the atomic species present in the system.")),
    "-outfile": dict(default="vasp.{}",type=str,
                        help=("Override the default output file names: 'vasp.{structure#}'" 
                              "for the structures.")),
    "-rattle": dict(default=0.0, type=float,
                        help=("Randomizes the positions of the atoms in the POSCAR by no "
                              "more than the fraction of the displacement provided."))
}
"""dict: default command-line arguments and their
    :meth:`argparse.ArgumentParser.add_argument` keyword arguments.
"""

def _parser_options():
    """Parses the options and arguments from the command line."""
    #We have two options: get some of the details from the config file,
    import argparse
    from phenum import base
    pdescr = "POSCAR contstruction."
    parser = argparse.ArgumentParser(parents=[base.bparser], description=pdescr)
    for arg, options in script_options.items():
        parser.add_argument(arg, **options)
        
    args = base.exhandler(examples, parser)
    if args is None:
        return

    return args #pragma: no cover

def run(args):
    """Generates the vasp output file for the desired structure.
    """

    if args == None:
        exit()
    if args["structures"] != None :
        if not RepresentsInt(args["structures"][0]) and args["structures"][0].lower() == "all":
            args["structures"] = None
        elif len(args["structures"])  == 1 and RepresentsInt(args["structures"][0]):
            args["structures"] = [int(args["structures"][0])]
        elif len(args["structures"]) == 2:
            args["structures"] = list(range(int(args["structures"][0]),
                                            int(args["structures"][1])+1))
        else:
            raise ValueError("Please enter a single structure number, two structures that "
                             "indicate the first and last structure to be used in the input "
                             "file, or all. The values {} don't match this "
                             "format.".format(args["structures"]))
    else:
        raise ValueError("Please enter a single structure number, two structures that "
                         "indicate the first and last structure to be used in the input "
                         "file, or all. The values {} don't match this "
                         "format.".format(args["structures"]))

    _make_structures(args)
        
if __name__ == '__main__':
    run(_parser_options())



#!/Users/trunks/envs/py3/bin/python3.4
from phenum import msg

"""The methods needed to take input from the user and put it in useful
forms. Also the actual executable for the code."""
# hard coded error tolerance. This will need to go away and become
# part of the input files later.

def _plot_HNFs(args,testmode=False):
    """Makes plots of the shapes of the HNF's provided in the enum.in input file.

    :arg args: The command line inputs.
    """
    if args["shapes"]:
        from phenum.visualize import HNF_shapes
        HNF_shapes(args["input"],args["lattice"],args["show"],testmode=testmode)
    else:
        from phenum.visualize import HNF_atoms
        HNF_atoms(args["input"],args["lattice"],args["show"],testmode=testmode)    

def _enum_in(args):
    """Makes an enum.in file that contains the desired distribution of
    structures from the polya distribution.

    :arg args: The command line inputs.
    """

    from phenum.structures import make_enum_in

    distribution = args["distribution"][0].lower()
    if args["sizes"]:
        sizes = list(range(*args["sizes"]))
    else:
        sizes = None
    if args["distribution"][1].lower() == "all":
        make_enum_in(distribution,args["cellsdir"],dataformat=args["dataformat"],sizes=sizes,
                     outfile=args["outfile"],save= True if args["savedist"] else False,seed=args["seed"],
                     restrict=args["filter"])
    else:
        make_enum_in(distribution,args["cellsdir"],number=int(args["distribution"][1]),
                     dataformat=args["dataformat"],sizes=sizes,outfile=args["outfile"],
                     save= True if args["savedist"] else False,seed=args["seed"],restrict=args["filter"])

def _polya_out(args):
    """Generates the 'polya.out' files for the cell sizes specified in 'lattice.in'
    (or other specified input file).
    """
    from os import path, makedirs
    from phenum.grouptheory import get_sym_group, get_full_HNF
    from phenum.msg import warn
    from phenum.symmetry import get_concs_for_size
    from phenum.io_utils import read_lattice, read_group
    from phenum.structures import enum_data
    import phenum.phonons as pb
    from phenum.polyaburnside import polya
    params = read_lattice(args["lattice"])

    for s in range(params["sizes"][0], params["sizes"][1]+1):
        celldir = "{}{}".format(args["cellsdir"],args["dataformat"].format(s))
        # get HNFs, SNFs, and LTs
        edata = enum_data(s,args,params)
        out = open(path.join(celldir, args["outfile"]), 'w+')
            
        # find the concentrations available for the desired cell sizes.
        cList = get_concs_for_size(s, params["nspecies"], params["is_crestricted"],
                                   len(params["basis_vecs"]), params["concs"])
        
        # We need to write the concs in cList to the output file.
        out.write('{0: <28}'.format("# HNF"))
        for conc in cList:
            out.write("{0: <50}".format(':'.join(map(str, conc))))
        out.write('{0: <50}\n'.format("Total"))

        a_concs = pb.get_arrow_concs(params)
        conc_totals = [0 for i in range(len(cList))]
        for idata, edict in enumerate(edata):
            HNF = edict["HNF"]
            out.write("  {0: <26}".format(' '.join(map(str, HNF))))

            if params["arrows"]:
                sym_g = get_sym_group(params["lat_vecs"],params["basis_vecs"],get_full_HNF(HNF),3)
                agroup = []
                for i in range(len(sym_g.perm.site_perm)):
                    agroup.append([sym_g.perm.site_perm[i],sym_g.perm.arrow_perm[i]])
            else:
                group = read_group(edict["group"])
                agroup = [[g,[0]] for g in group]

            # we need to loop over the concentrations and find the
            # number of arrangements possible for each cell size
            total = 0
            for iconc, conc in enumerate(cList):
                if len(conc) > 0:

                    decorations = pb.arrow_concs(conc,a_concs)
                
                    # we need to know the concentrations of the
                    # species with and without arrows, we also need to
                    # know the number of arrows and their species so
                    # we can undo the previous step later
                    (n_arrows,arrow_types,concs_w_arrows) = pb.how_many_arrows(decorations)

                    # now find the number of unique arrangements using Polya.
                    if arrow_types != 0:
                        # Since enumlib doesn't write the arrow group
                        # out we have to recompute the group actions
                        # paired with their effects on the arrows
                        total_num = polya(concs_w_arrows,agroup,arrowings=arrow_types)
                    else:
                        total_num = polya(conc, agroup)

                    out.write("{0: <50d}".format(total_num))
                    total += total_num
                    conc_totals[iconc] += total_num
            out.write('{0: <10d}\n'.format(total))
        out.write("# " + ''.join(['-' for i in range(len(cList)*10 + 10 + 30)]) + '\n')
        out.write("{0: <28}".format("  0 0 0 0 0 0"))
        for ctotal in conc_totals:
            out.write("{0: <50d}".format(ctotal))
        out.write("{0: <50d}\n".format(sum(conc_totals)))
        out.close()
        
# check which files are present to see if we are finding the HNFs with
# the polya count or enumerating subsets within the HNFs.
def _enum_out(args):
    """Produce the enumerations of a subset of the total number of possible 
    arrangements for the desired HNFs. It assumes that all the information 
    used in the polya part above is still available.
    """

    import phenum.io_utils as io
    from phenum.structures import enum_data
    import phenum.phonons as pb
    from numpy import unique
    from phenum.grouptheory import get_full_HNF, SmithNormalForm

    keep_supers = False
    if args["super"]:
        keep_supers = True
    
    params = io.read_lattice(args["lattice"])
    systems = io.read_enum(args["input"])
    io.write_enum(params, outfile=args["outfile"])    

    count_t = 1
    count_s = 0
    from operator import itemgetter
    def cellsize(sHNF):
        return sHNF[0]*sHNF[2]*sHNF[5]
    cellsizes = unique([cellsize(sys[0]) for sys in systems])
    datadicts = {}
    sfmt = ("{0: >10d}{1: >10d}{2: >8d}{3: >9d}{4: >9d}{5: >12d}{6: >4d}{7: >6d}"
            "{8: >10}  {9: >18}  {10: >44}    {11}    {12: >21}\n")
    def fmtn(l, n):
        return (''.join(["{{{0:d}: >{1:d}d}}".format(i, n) for i in range(len(l))])).format(*l)
    
    with open(args["outfile"], 'a') as f:
        if not params["arrows"]:
            for s in cellsizes:
                dataset = enum_data(s,args,params)
                datadicts.update({tuple(d["HNF"]): d for d in dataset})

        enumlist = []    
        for HNF, conc, num_wanted in systems:
            if not params["arrows"]:
                edata = datadicts[tuple(HNF)]
                SNF = edata["SNF"]
                LT = edata["L"]
            
                a_concs = pb.get_arrow_concs(params)
                configs = pb.enum_sys(edata["group"], list(conc), a_concs, num_wanted,HNF,params,
                                      keep_supers, args["acceptrate"])
            else:
                (SNF,L,R) = SmithNormalForm(get_full_HNF(HNF))
                SNF = [SNF[0][0],SNF[1][1],SNF[2][2]]
                LT = [item for row in L for item in row]
                a_concs = pb.get_arrow_concs(params)
                configs = pb.enum_sys(None, list(conc), a_concs, num_wanted,HNF,params, keep_supers, args["acceptrate"])

            for config in configs:
                labeling, arrowing = io.create_labeling(config)
                enumlist.append((sum(conc), HNF, SNF, LT, labeling, arrowing))
                
            sortenum = sorted(enumlist, key=itemgetter(0, 4))
            if len(sortenum) > 0:
                last_sz = sortenum[0][0]
            
        for (size, HNF, SNF, LT, labeling, arrowing) in sortenum:
            if size != last_sz:
                count_s = 1
                last_sz = size
            else:
                count_s += 1
            o = sfmt.format(count_t, 1, 1, 1, 1, count_s, size, 1, fmtn(SNF, 3), fmtn(HNF, 3),
                            fmtn(LT, 5), labeling, arrowing)
            f.write(o)
            count_t += 1

def examples():

    """Prints examples of using the script to the console using colored output.
    """
    script = "PHENUM: Finds the symmetrically unique arrangements of atoms in a lattice."
    explain = ("For simple 1D potentials such as the infinite square well, " 
               "kronig-penny, ect. This code produces a numerical solution "
               "using a bisis expansion.")
    contents = [("Find the Polya distribution",
            "The code below finds the number of unique arrangements for each "
            "supercell (HNF) for a binary system on an fcc lattice which can have "
            "1 to 11 atoms in the supercell as described in the lattice.in found in "
            "input/fcc sample directory. The files labeled cell.n contain the HNFs "
            "and symmetry group for the supercells of size n and were generated by "
            "the modified enum.x code. For more information on the contents of this "
            "folder please see the README. To a different input file to use rather "
            "than lattice.in use the -lattice option or if you have the HNF and "
            "symmetry group data in a different file then cells.n then use "
            "-dataformat.","enumeration.py -polya"),
           ("Construct an enum.in file before running the -enum mode",
            "This code assumes that the -polya mode has arleady been run. It takes two"
            " arguments; the first is the disered distribution type ('size', 'conc',"
            " 'shape', 'all'), the second is the desired number of structures, if all"
            " the structures are wanted then the second argument should be 'all'.",
            "enumeration.py -distribution all all \n  enumeration.py -distribution "
            " size 200"),
           ("Find the desired number of unique structures",
            "This code also uses the sample system and files found in "
            "input/fcc. It finds the desired number of unique arrangements for each "
            "HNF and concentration range listed in the enum.in file. This file has "
            "the format:\n # HNF                           Conc.       Number\n"
            "   1 0 1 0 0 4                   1 3         1\n "
            "Where HNF is the lower triangle of the HNF Conc is the concentrations "
            "of the 2 atoms on the lattice and Number specifies the number of "
            "arrangements for that HNF and concentration range the user desires."
            , "enumeration.py -enum")]

    required = ("REQUIRED: the system to be enumerated in a `enum.in` file "
                "and a `lattice.in` file that contains the lattice data.")
    output = ("RETURNS: a file `struct_enum.out` for the full enumeration, "
              "an `enum.in` file if the -distribution flag is used, or a "
              "`polya.out` file for each cell size if the -polya flag is used.")
    details = ("For all the examples below, it is assumed that you have already "
                "compiled the modified enumlib code as described in the "
                "README or in some other manner obtained the HNFs (supercells) and "
                "their symmetry groups. You will then need to specify if you are "
                "obtaining the number of unique arrangements for each supercell and "
                "concentration allowed for your system or enumerating (finding) the "
                "desired number of configurations for each HNF and concentration. "
                "Additionally you way change the default input file names to ones of "
                "your own creation.")
    outputfmt = ("")

    msg.example(script, explain, contents, required, output, outputfmt, details)

script_options = {
    "-polya": dict(action="store_true",
                   help=("Predict the total number of unique superstructures for each "
                         "cell shape and possible concentration.")),
    "-enum": dict(action="store_true",
                  help=("Enumerate the number of superstructures of specific shape "
                        "as specified in the 'enum.in' file.")),
    "-lattice": dict(default="lattice.in", type=str,
                     help=("Override the default input file name: 'lattice.in' for "
                           "enumeration parameters.")),
    "-input": dict(default="enum.in", type=str,
                   help=("Override the default 'enum.in' file name.")),
    "-exec": dict(default="enum.x", type=str,
                  help=("Override the default 'enum.x' executable with an executable "
                        "name or path.")),
    "-distribution": dict(default=None, nargs= "+", 
                        help=("Makes an enum.in file when the distribution is 'all'. Otherwise "
                              "the distribution is printed to the screen for the user. "
                              " The distributions are built from the results of the polya "
                              "run and contain the specified number of structures.")),
    "-savedist": dict(action="store_true",
                      help=("Makes an enum.in file for the distribution over 'size', "
                            "'shape', or 'conc'.")),
    "-dataformat": dict(default="cells.{}", type=str,
                        help=("Specify the default folder name for any cell size that contains "
                              "the matrices and groups generated by 'enum.x'. Format is: cells.{} "
                              "where {} is a placeholder for the integer cell size.")),
    "-cellsdir": dict(default="", type=str,
                      help=("Specify the path to the cells.{} type folders for the run. This "
                            "can be a relative or an absolute path.")),
    "-outfile": dict(default=None, type=str,
                     help=("Override the default output file names: 'polya.out' for "
                           "polya counting; 'enum.out' for structure enumeration; "
                           "'enum.in' for the distributions.")),
    "-sizes": dict(default= None, nargs=2, type=int,
                   help=("Specify the start and stop cell sizes over which to distribute "
                         "the structure selection, weighted by the Polya distribution.")),
    "-acceptrate": dict(default=None, type=float,
                        help=("For large numbers of unique arrangements, specify how often "
                              "one should be kept in the final list.")),
    "-profile": dict(action="store_true",
                     help=("Profile the code as it runs. Requires vprof (`pip install "
                           "vprof`). Argument is a combination of 'c', 'm' and 'h'. See "
                           "vprof docs.")),
    "-seed": dict(default=None, type=int,
                  help=("The integer seed for the random number generator.")),
    
    "-super": dict(action="store_true",
                   help=("Overrides the exclusion of the superperiodic cells from the output.")),
    "-filter": dict(default=None, nargs = 2,
                    help=("Applys a filter over the 'shape' or the 'conc' option of the "
                          "distributions. The first entry should specify the filter the "
                          "second should be the name of the file containing the desired "
                          "restrictions.")),
    "-visualize": dict(action="store_true",
                       help=("Makes plots of the shapes of the HNF's in the provided "
                             "enum.in style file.")),
    "-shapes": dict(action="store_true",
                    help=("The visualization excludes the atoms showing just the shapes "
                          "of the cells.")),
    "-show": dict(action="store_true",
                  help=("If present then the visualization will loop through an interactive "
                        "image for each HNF.")),
}
"""dict: default command-line arguments and their
    :meth:`argparse.ArgumentParser.add_argument` keyword arguments.
"""

def _parser_options():
    """Parses the options and arguments from the command line."""
    #We have two options: get some of the details from the config file,
    import argparse
    from phenum import base
    pdescr = "Numerical DFT code."
    parser = argparse.ArgumentParser(parents=[base.bparser], description=pdescr)
    for arg, options in script_options.items():
        parser.add_argument(arg, **options)
        
    args = base.exhandler(examples, parser)
    if args is None:
        return

    return args # pragma: no cover

def _script_enum(args, testmode=False):
    """Generates the 'polya.out' or 'enum.out' files depending on the script arguments.
    """
    from os import path, system
    from phenum.base import set_testmode
    set_testmode(testmode)
    if args["polya"]:
        #Perform validation for running polya.
        if args["outfile"] == None:
              args["outfile"] = "polya.out"
        if not path.isfile(args["lattice"]):
            from phenum.msg import err
            from os import listdir
            err("The input file {} does not exist.".format(args["lattice"]))
            exit()
            
    if args["enum"]:
        if args["outfile"] == None:
              args["outfile"] = "enum.out"
        #Perform validation for running enum.
        if not path.isfile(args["input"]):
            from phenum.msg import err
            err("The input file {} does not exist.".format(args["input"]))
            exit()

    if args["enum"] or args["polya"]:
        from glob import glob
        #Perform validation for running enum.
        if len(glob("{}{}".format(args["cellsdir"],args["dataformat"].split('.')[0]+'.*'))) < 1:
            from phenum.msg import err, warn
            from phenum.io_utils import which, read_lattice, write_struct_enum
            from os import system, chdir, getcwd
            warn("The input folders {}{} do not exist.".format(args["cellsdir"],args["dataformat"]))
            warn("Now running your enumlib executable to build the folders.")
            # We need to know where the current directory is so we can get back to it.
            initial_dir = getcwd()
            if args["cellsdir"] != "":
                system("cp {} {}.".format(args["lattice"],args["cellsdir"]))
                if args["enum"]:
                    system("cp {} {}.".format(args["input"],args["cellsdir"]))

                chdir(args["cellsdir"])
                if path.isfile("struct_enum.out"):
                    system("rm struct_enum.out")
                
            if which(args["exec"]) != None:
                write_struct_enum(read_lattice(args["lattice"]))
                system(args["exec"])
                system('rm symops_enum_parent_lattice.out readcheck_enum.out fort.*')
                if len(glob(args["dataformat"].split('.')[0]+'.*')) < 1:
                    err("The executable you have for {} does not produce "
                        "the needed input folders {}. In order to correct this "
                        "you need to follow the compilation instructions found "
                        "in the README.".format(args["exec"],args["dataformat"]))
                    exit()                            
            else:
                err("Could not find {} on your path. Please add it to your path "
                    "if you have already\n compiled it. Otherwise please follow the "
                    "instructions found in the README to download, make,\n and place the "
                    "executable in your path.".format(args["exec"]))
                exit()
            # Navigate back to the initial directory if we left it.
            if initial_dir != getcwd():
                chdir(initial_dir)

    if args["distribution"]:
        if args["outfile"] == None:
              args["outfile"] = "enum.in"
        if len(args["distribution"]) != 2:
            from phenum.msg import err
            err("The distribution option takes two arguments. The parameters the distribution "
                " is over ('shape', 'conc', 'size', 'all') and the number of structures desired."
                "If all the structures are wanted then the second argument should be 'all'.")
            exit()
            
    if args["polya"]:
        _polya_out(args)
    if args["enum"]:
        _enum_out(args)
    if args["distribution"]:
        _enum_in(args)
    if args["visualize"]:
        _plot_HNFs(args,testmode=testmode)
        
if __name__ == '__main__':
    args = _parser_options()
    
    if args["profile"]:
        from vprof import profiler
        profiler.run(_script_enum, args["profile"], args=(args,), host='localhost', port=8000)
    else:
        _script_enum(args)
